import _typeshed
from collections import OrderedDict
from collections.abc import Iterator, Sequence
from functools import partial
from logging import Logger
from typing import Any, ClassVar, Generic, Literal, NoReturn, TypeVar, overload
from typing_extensions import TypeAlias, deprecated

from django.db.models import Field as DjangoField, Model, QuerySet
from django.utils.safestring import SafeString

from .declarative import DeclarativeMetaclass, ModelDeclarativeMetaclass
from .fields import Field
from .instance_loaders import BaseInstanceLoader
from .options import ResourceOptions
from .results import Error, Result, RowResult
from .widgets import ForeignKeyWidget, ManyToManyWidget, Widget

Dataset: TypeAlias = _typeshed.Incomplete  # tablib.Dataset
logger: Logger

def has_natural_foreign_key(model: Model) -> bool: ...

class Diff:
    left: list[str]
    right: list[str]
    new: bool
    def __init__(self, resource: Resource[_ModelT], instance: _ModelT, new: bool) -> None: ...
    def compare_with(self, resource: Resource[_ModelT], instance: _ModelT, dry_run: bool = False) -> None: ...
    def as_html(self) -> list[SafeString]: ...

_ModelT = TypeVar("_ModelT", bound=Model)

class Resource(Generic[_ModelT], metaclass=DeclarativeMetaclass):
    _meta: ResourceOptions[_ModelT]
    fields: OrderedDict[str, Field]
    create_instances: list[_ModelT]
    update_instances: list[_ModelT]
    delete_instances: list[_ModelT]
    def __init__(self, **kwargs: Any) -> None: ...
    @classmethod
    def get_result_class(self) -> type[Result]: ...
    @classmethod
    def get_row_result_class(self) -> type[RowResult]: ...
    @classmethod
    def get_error_result_class(self) -> type[Error]: ...
    @classmethod
    def get_diff_class(self) -> type[Diff]: ...
    @classmethod
    def get_db_connection_name(self) -> str: ...
    def get_use_transactions(self) -> bool: ...
    def get_chunk_size(self) -> int: ...
    @deprecated("The 'get_fields()' method is deprecated and will be removed in a future release.")
    def get_fields(self, **kwargs: Any) -> list[Field]: ...
    def get_field_name(self, field: Field) -> str: ...
    def init_instance(self, row: dict[str, Any] | None = None) -> _ModelT: ...
    def get_instance(self, instance_loader: BaseInstanceLoader, row: dict[str, Any]) -> _ModelT | None: ...
    def get_or_init_instance(self, instance_loader: BaseInstanceLoader, row: dict[str, Any]) -> tuple[_ModelT | None, bool]: ...
    def get_import_id_fields(self) -> Sequence[str]: ...
    def get_bulk_update_fields(self) -> list[str]: ...
    def bulk_create(
        self,
        using_transactions: bool,
        dry_run: bool,
        raise_errors: bool,
        batch_size: int | None = None,
        result: Result | None = None,
    ) -> None: ...
    def bulk_update(
        self,
        using_transactions: bool,
        dry_run: bool,
        raise_errors: bool,
        batch_size: int | None = None,
        result: Result | None = None,
    ) -> None: ...
    def bulk_delete(self, using_transactions: bool, dry_run: bool, raise_errors: bool, result: Result | None = None) -> None: ...
    def validate_instance(
        self, instance: _ModelT, import_validation_errors: dict[str, Any] | None = None, validate_unique: bool = True
    ) -> None: ...
    # For all the definitions below (from `save_instance()` to `import_row()`), `**kwargs` should contain:
    # dry_run: bool, use_transactions: bool, row_number: int, retain_instance_in_row_result: bool.
    # Users are free to pass extra arguments in `import_data()`so PEP 728 can probably be leveraged here.
    def save_instance(self, instance: _ModelT, is_create: bool, row: dict[str, Any], **kwargs: Any) -> None: ...
    def do_instance_save(self, instance: _ModelT) -> None: ...
    def before_save_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def after_save_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def delete_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def before_delete_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def after_delete_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def import_field(self, field: Field, instance: _ModelT, row: dict[str, Any], is_m2m: bool = False, **kwargs: Any) -> None: ...
    def get_import_fields(self) -> list[Field]: ...
    def import_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def save_m2m(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def for_delete(self, row: dict[str, Any], instance: _ModelT) -> bool: ...
    def skip_row(
        self, instance: _ModelT, original: _ModelT, row: dict[str, Any], import_validation_errors: dict[str, Any] | None = None
    ) -> bool: ...
    def get_diff_headers(self) -> list[str]: ...
    def before_import(self, dataset: Dataset, **kwargs: Any) -> None: ...
    def after_import(self, dataset: Dataset, result: Result, **kwargs: Any) -> None: ...
    def before_import_row(self, row: dict[str, Any], **kwargs: Any) -> None: ...
    def after_import_row(self, row: dict[str, Any], row_result: RowResult, **kwargs: Any) -> None: ...
    def after_init_instance(self, instance: _ModelT, new: bool, row: dict[str, Any], **kwargs: Any) -> None: ...
    @overload
    def handle_import_error(self, result: Result, error: Exception, raise_errors: Literal[True]) -> NoReturn: ...
    @overload
    def handle_import_error(self, result: Result, error: Exception, raise_errors: Literal[False] = False) -> None: ...
    def import_row(self, row: dict[str, Any], instance_loader: BaseInstanceLoader, **kwargs: Any) -> RowResult: ...
    def import_data(
        self,
        dataset: Dataset,
        dry_run: bool = False,
        raise_errors: bool = False,
        use_transactions: bool | None = None,
        collect_failed_rows: bool = False,
        rollback_on_validation_errors: bool = False,
        **kwargs: Any,
    ) -> Result: ...
    def import_data_inner(
        self,
        dataset: Dataset,
        dry_run: bool,
        raise_errors: bool,
        using_transactions: bool,
        collect_failed_rows: bool,
        **kwargs: Any,
    ) -> Result: ...
    def get_import_order(self) -> tuple[str, ...]: ...
    def get_export_order(self) -> tuple[str, ...]: ...
    def before_export(self, queryset: QuerySet[_ModelT], **kwargs: Any) -> None: ...
    def after_export(self, queryset: QuerySet[_ModelT], dataset: Dataset, **kwargs: Any) -> None: ...
    def filter_export(self, queryset: QuerySet[_ModelT], **kwargs: Any) -> QuerySet[_ModelT]: ...
    def export_field(self, field: Field, instance: _ModelT, **kwargs: Any) -> str: ...
    def get_export_fields(self, selected_fields: Sequence[str] | None = None) -> list[Field]: ...
    def export_resource(self, instance: _ModelT, selected_fields: Sequence[str] | None = None, **kwargs: Any) -> list[str]: ...
    def get_export_headers(self, selected_fields: Sequence[str] | None = None) -> list[str]: ...
    def get_user_visible_headers(self) -> list[str]: ...
    def get_user_visible_fields(self) -> list[str]: ...
    def iter_queryset(self, queryset: QuerySet[_ModelT]) -> Iterator[_ModelT]: ...
    def export(self, queryset: QuerySet[_ModelT] | None = None, **kwargs: Any) -> Dataset: ...

class ModelResource(Resource[_ModelT], metaclass=ModelDeclarativeMetaclass):
    DEFAULT_RESOURCE_FIELD: ClassVar[type[Field]] = ...
    WIDGETS_MAP: ClassVar[dict[str, type[Widget]]]
    @classmethod
    def get_m2m_widget(cls, field: DjangoField[Any, Any]) -> partial[ManyToManyWidget[Any]]: ...
    @classmethod
    def get_fk_widget(cls, field: DjangoField[Any, Any]) -> partial[ForeignKeyWidget[Any]]: ...
    @classmethod
    def widget_from_django_field(cls, f: DjangoField[Any, Any], default: type[Widget] = ...) -> type[Widget]: ...
    @classmethod
    def widget_kwargs_for_field(cls, field_name: str, django_field: DjangoField[Any, Any]) -> dict[str, Any]: ...
    @classmethod
    def field_from_django_field(cls, field_name: str, django_field: DjangoField[Any, Any], readonly: bool) -> Field: ...
    def get_queryset(self) -> QuerySet[_ModelT]: ...
    def init_instance(self, row: dict[str, Any] | None = None) -> _ModelT: ...
    def after_import(self, dataset: Dataset, result: Result, **kwargs: Any) -> None: ...
    @classmethod
    def get_display_name(cls) -> str: ...

_ResourceT = TypeVar("_ResourceT", bound=Resource[Any])

# HK Type Vars could help type the first overload:
@overload
def modelresource_factory(model: Model, resource_class: type[_ResourceT]) -> _ResourceT: ...
@overload
def modelresource_factory(model: _ModelT) -> ModelResource[_ModelT]: ...



from typing import Any
from typing_extensions import Self, TypeAlias

__tracebackhide__: bool

_IncludeIgnore: TypeAlias = str | list[str] | list[tuple[str, ...]] | None

class BaseMixin:
    description: str
    def described_as(self, description: str) -> Self: ...
    def is_equal_to(self, other: Any, *, include: _IncludeIgnore = None, ignore: _IncludeIgnore = None) -> Self: ...
    def is_not_equal_to(self, other: Any) -> Self: ...
    def is_same_as(self, other: Any) -> Self: ...
    def is_not_same_as(self, other: Any) -> Self: ...
    def is_true(self) -> Self: ...
    def is_false(self) -> Self: ...
    def is_none(self) -> Self: ...
    def is_not_none(self) -> Self: ...
    def is_type_of(self, some_type: type) -> Self: ...
    def is_instance_of(self, some_class: type) -> Self: ...
    def is_length(self, length: int) -> Self: ...



from __future__ import annotations

from dateutil import relativedelta


# An illustrative example for why we re-export dateutil._common.weekday from dateutil.relativedelta in the stub
class Calendar:
    def __init__(self, week_start: relativedelta.weekday = relativedelta.MO) -> None:
        self.week_start = week_start



from _typeshed import Incomplete

from tensorflow._aliases import IntArray, IntDataSequence

Layout = Incomplete

class Mesh:
    def __init__(
        self,
        dim_names: list[str],
        global_device_ids: IntArray | IntDataSequence,
        local_device_ids: list[int],
        local_devices: list[Incomplete | str],
        mesh_name: str = "",
        global_devices: list[Incomplete | str] | None = None,
        use_xla_spmd: bool = False,
    ) -> None: ...

def __getattr__(name: str): ...  # incomplete module



from typing import overload

import numpy as np

from ._typing import ArrayLike, GeoArray, GeoArrayLikeSeq, OptGeoArrayLike
from .geometry import Polygon

__all__ = ["coverage_invalid_edges", "coverage_is_valid", "coverage_simplify"]

def coverage_is_valid(geometry: OptGeoArrayLike, gap_width: float = 0.0, **kwargs) -> np.bool_: ...
def coverage_invalid_edges(geometry: OptGeoArrayLike, gap_width: float = 0.0, **kwargs) -> GeoArray: ...
@overload
def coverage_simplify(geometry: Polygon, tolerance: ArrayLike[float], *, simplify_boundary: bool = True) -> Polygon: ...
@overload
def coverage_simplify(geometry: GeoArrayLikeSeq, tolerance: ArrayLike[float], *, simplify_boundary: bool = True) -> GeoArray: ...



from _typeshed import Unused
from collections.abc import Callable

from aws_xray_sdk.core.models.entity import Entity
from aws_xray_sdk.core.models.segment import Segment

class DefaultStreaming:
    def __init__(self, streaming_threshold: int = 30) -> None: ...
    def is_eligible(self, segment: Segment) -> bool: ...
    def stream(self, entity: Entity, callback: Callable[..., Unused]) -> None: ...
    @property
    def streaming_threshold(self) -> int: ...
    @streaming_threshold.setter
    def streaming_threshold(self, value: int) -> None: ...



from _typeshed import Incomplete
from abc import ABC, abstractmethod

S_OK: int
IDispatchType: Incomplete
IUnknownType: Incomplete
regSpec: str
regPolicy: str
regDispatcher: str
regAddnPath: str

def CreateInstance(clsid, reqIID): ...

class BasicWrapPolicy(ABC):
    def __init__(self, object) -> None: ...
    def _InvokeEx_(self, dispid, lcid, wFlags, args, kwargs, serviceProvider) -> tuple[Incomplete]: ...
    @abstractmethod
    def _invokeex_(self, dispid, lcid, wFlags, args, kwargs, serviceProvider) -> tuple[Incomplete]: ...

class MappedWrapPolicy(BasicWrapPolicy):
    _dispid_to_func_: dict[int, str]
    def _invokeex_(self, dispid, lcid, wFlags, args, kwargs, serviceProvider) -> tuple[Incomplete]: ...

class DesignatedWrapPolicy(MappedWrapPolicy): ...
class EventHandlerPolicy(DesignatedWrapPolicy): ...

class DynamicPolicy(BasicWrapPolicy):
    def _invokeex_(self, dispid, lcid, wFlags, args, kwargs, serviceProvider) -> tuple[Incomplete]: ...

DefaultPolicy = DesignatedWrapPolicy

def resolve_func(spec): ...
def call_func(spec, *args): ...

DISPATCH_METHOD: int
DISPATCH_PROPERTYGET: int
DISPATCH_PROPERTYPUT: int
DISPATCH_PROPERTYPUTREF: int
DISPID_EVALUATE: int
DISPID_NEWENUM: int
DISPID_PROPERTYPUT: int
DISPID_STARTENUM: int
DISPID_VALUE: int



from typing import Literal, overload

@overload
def str2bool(value: str, raise_exc: Literal[True]) -> bool: ...
@overload
def str2bool(value: str, raise_exc: bool = False) -> bool | None: ...
def str2bool_exc(value: str) -> bool: ...



from _typeshed import Incomplete
from typing import Final

DEFAULTS: dict[str, str | bool | tuple[Incomplete] | list[Incomplete] | None]
XRAY_NAMESPACE: Final = "XRAY_RECORDER"
SUPPORTED_ENV_VARS: tuple[str, ...]

class XRaySettings:
    defaults: dict[str, str | bool | tuple[Incomplete] | list[Incomplete] | None]
    def __init__(self, user_settings=None) -> None: ...
    @property
    def user_settings(self): ...
    def __getattr__(self, attr): ...

settings: XRaySettings

def reload_settings(*, settings: str | None = None, value=None) -> None: ...



from collections.abc import Callable, Collection, Iterable, Mapping
from typing import Any, TypeVar, overload
from typing_extensions import TypeAlias

from ._utils import NO_DEFAULT, ExtractorError

_Traversable: TypeAlias = Mapping[str, Any] | Iterable[Any]
_PathArg: TypeAlias = str | int

def traverse_obj(
    obj: _Traversable,
    *paths: _PathArg,
    default: Any = ...,  # Anything or type[NO_DEFAULT]
    expected_type: type[Any] | None = None,
    get_all: bool = True,
    casesense: bool = True,
    is_user_input: bool | type[NO_DEFAULT] = ...,
    traverse_string: bool = False,
) -> Any: ...  # Unknown return type

_T = TypeVar("_T")

def value(value: _T, /) -> _T: ...
def require(name: str, /, *, expected: bool = False) -> Callable[[_T], _T]: ...

class _RequiredError(ExtractorError): ...

@overload
def subs_list_to_dict(
    *, lang: str | None = "und", ext: str | None = None
) -> Callable[[list[dict[str, Any]]], dict[str, list[dict[str, Any]]]]: ...
@overload
def subs_list_to_dict(
    subs: list[dict[str, Any]] | None, /, *, lang: str | None = "und", ext: str | None = None
) -> dict[str, list[dict[str, Any]]]: ...
@overload
def find_element(*, attr: str, value: str, tag: str | None = None, html: bool = False, regex: bool = False) -> str: ...
@overload
def find_element(*, cls: str, html: bool = False) -> str: ...
@overload
def find_element(*, id: str, tag: str | None = None, html: bool = False, regex: bool = False) -> str: ...
@overload
def find_element(*, tag: str, html: bool = False, regex: bool = False) -> str: ...
@overload
def find_element(
    *,
    tag: str | None = None,
    id: str | None = None,
    cls: str | None = None,
    attr: str | None = None,
    value: str | None = None,
    html: bool = False,
    regex: bool = False,
) -> str: ...
@overload
def find_elements(*, cls: str, html: bool = False) -> list[str]: ...
@overload
def find_elements(*, attr: str, value: str, tag: str | None = None, html: bool = False, regex: bool = False) -> list[str]: ...
@overload
def find_elements(
    *,
    tag: str | None = None,
    cls: str | None = None,
    attr: str | None = None,
    value: str | None = None,
    html: bool = False,
    regex: bool = False,
) -> list[str]: ...
def trim_str(*, start: str | None = None, end: str | None = None) -> Callable[[str], str]: ...

# Returns a callable f(items) which calls func(*items, **kwargs).
def unpack(func: Callable[..., Any], **kwargs: Any) -> Callable[..., Any]: ...
def get_first(
    obj: _Traversable,
    *paths: _PathArg,
    default: Any = ...,  # Anything or type[NO_DEFAULT]
    expected_type: type[Any] | None = None,
    get_all: bool = True,
    casesense: bool = True,
    is_user_input: bool | type[NO_DEFAULT] = ...,
    traverse_string: bool = False,
) -> Any: ...
@overload
def dict_get(d: str, key_or_keys: str | Collection[str]) -> Any | None: ...
@overload
def dict_get(
    d: str, key_or_keys: str | Collection[str], default: Any | None = None, skip_false_values: bool = True
) -> Any | None: ...



from ._deprecated import (
    bytes_to_intlist as bytes_to_intlist,
    compiled_regex_type as compiled_regex_type,
    intlist_to_bytes as intlist_to_bytes,
    jwt_encode_hs256 as jwt_encode_hs256,
)
from ._utils import (
    ACCENT_CHARS as ACCENT_CHARS,
    BOMS as BOMS,
    DATE_FORMATS as DATE_FORMATS,
    DATE_FORMATS_DAY_FIRST as DATE_FORMATS_DAY_FIRST,
    DATE_FORMATS_MONTH_FIRST as DATE_FORMATS_MONTH_FIRST,
    DEFAULT_OUTTMPL as DEFAULT_OUTTMPL,
    DOT_DESKTOP_LINK_TEMPLATE as DOT_DESKTOP_LINK_TEMPLATE,
    DOT_URL_LINK_TEMPLATE as DOT_URL_LINK_TEMPLATE,
    DOT_WEBLOC_LINK_TEMPLATE as DOT_WEBLOC_LINK_TEMPLATE,
    ENGLISH_MONTH_NAMES as ENGLISH_MONTH_NAMES,
    IDENTITY as IDENTITY,
    JSON_LD_RE as JSON_LD_RE,
    KNOWN_EXTENSIONS as KNOWN_EXTENSIONS,
    LINK_TEMPLATES as LINK_TEMPLATES,
    MEDIA_EXTENSIONS as MEDIA_EXTENSIONS,
    MONTH_NAMES as MONTH_NAMES,
    NO_DEFAULT as NO_DEFAULT,
    NUMBER_RE as NUMBER_RE,
    OUTTMPL_TYPES as OUTTMPL_TYPES,
    PACKED_CODES_RE as PACKED_CODES_RE,
    POSTPROCESS_WHEN as POSTPROCESS_WHEN,
    STR_FORMAT_RE_TMPL as STR_FORMAT_RE_TMPL,
    STR_FORMAT_TYPES as STR_FORMAT_TYPES,
    TIMEZONE_NAMES as TIMEZONE_NAMES,
    TV_PARENTAL_GUIDELINES as TV_PARENTAL_GUIDELINES,
    US_RATINGS as US_RATINGS,
    WINDOWS_VT_MODE as WINDOWS_VT_MODE,
    Config as Config,
    ContentTooShortError as ContentTooShortError,
    DateRange as DateRange,
    DownloadCancelled as DownloadCancelled,
    DownloadError as DownloadError,
    EntryNotInPlaylist as EntryNotInPlaylist,
    ExistingVideoReached as ExistingVideoReached,
    ExtractorError as ExtractorError,
    FormatSorter as FormatSorter,
    GeoRestrictedError as GeoRestrictedError,
    GeoUtils as GeoUtils,
    HTMLAttributeParser as HTMLAttributeParser,
    HTMLBreakOnClosingTagParser as HTMLBreakOnClosingTagParser,
    HTMLListAttrsParser as HTMLListAttrsParser,
    InAdvancePagedList as InAdvancePagedList,
    ISO639Utils as ISO639Utils,
    ISO3166Utils as ISO3166Utils,
    LazyList as LazyList,
    LenientJSONDecoder as LenientJSONDecoder,
    LockingUnsupportedError as LockingUnsupportedError,
    MaxDownloadsReached as MaxDownloadsReached,
    Namespace as Namespace,
    OnDemandPagedList as OnDemandPagedList,
    PagedList as PagedList,
    PlaylistEntries as PlaylistEntries,
    Popen as Popen,
    PostProcessingError as PostProcessingError,
    ReExtractInfo as ReExtractInfo,
    RegexNotFoundError as RegexNotFoundError,
    RejectedVideoReached as RejectedVideoReached,
    RetryManager as RetryManager,
    SameFileError as SameFileError,
    ThrottledDownload as ThrottledDownload,
    UnavailableVideoError as UnavailableVideoError,
    UnsupportedError as UnsupportedError,
    UserNotLive as UserNotLive,
    XAttrMetadataError as XAttrMetadataError,
    XAttrUnavailableError as XAttrUnavailableError,
    YoutubeDLError as YoutubeDLError,
    age_restricted as age_restricted,
    args_to_str as args_to_str,
    ass_subtitles_timecode as ass_subtitles_timecode,
    base_url as base_url,
    bool_or_none as bool_or_none,
    bug_reports_message as bug_reports_message,
    bytes_to_long as bytes_to_long,
    cached_method as cached_method,
    caesar as caesar,
    check_executable as check_executable,
    classproperty as classproperty,
    clean_html as clean_html,
    clean_podcast_url as clean_podcast_url,
    cli_bool_option as cli_bool_option,
    cli_configuration_args as cli_configuration_args,
    cli_option as cli_option,
    cli_valueless_option as cli_valueless_option,
    date_formats as date_formats,
    date_from_str as date_from_str,
    datetime_add_months as datetime_add_months,
    datetime_from_str as datetime_from_str,
    datetime_round as datetime_round,
    decode_base_n as decode_base_n,
    decode_packed_codes as decode_packed_codes,
    deprecation_warning as deprecation_warning,
    detect_exe_version as detect_exe_version,
    determine_ext as determine_ext,
    determine_file_encoding as determine_file_encoding,
    determine_protocol as determine_protocol,
    dfxp2srt as dfxp2srt,
    download_range_func as download_range_func,
    encode_base_n as encode_base_n,
    encode_compat_str as encode_compat_str,
    encode_data_uri as encode_data_uri,
    encodeArgument as encodeArgument,
    error_to_str as error_to_str,
    escapeHTML as escapeHTML,
    expand_path as expand_path,
    ext2mimetype as ext2mimetype,
    extract_attributes as extract_attributes,
    extract_basic_auth as extract_basic_auth,
    extract_timezone as extract_timezone,
    filesize_from_tbr as filesize_from_tbr,
    filter_dict as filter_dict,
    find_available_port as find_available_port,
    find_xpath_attr as find_xpath_attr,
    fix_xml_ampersands as fix_xml_ampersands,
    float_or_none as float_or_none,
    format_bytes as format_bytes,
    format_decimal_suffix as format_decimal_suffix,
    format_field as format_field,
    formatSeconds as formatSeconds,
    frange as frange,
    function_with_repr as function_with_repr,
    get_compatible_ext as get_compatible_ext,
    get_domain as get_domain,
    get_element_by_attribute as get_element_by_attribute,
    get_element_by_class as get_element_by_class,
    get_element_by_id as get_element_by_id,
    get_element_html_by_attribute as get_element_html_by_attribute,
    get_element_html_by_class as get_element_html_by_class,
    get_element_html_by_id as get_element_html_by_id,
    get_element_text_and_html_by_tag as get_element_text_and_html_by_tag,
    get_elements_by_attribute as get_elements_by_attribute,
    get_elements_by_class as get_elements_by_class,
    get_elements_html_by_attribute as get_elements_html_by_attribute,
    get_elements_html_by_class as get_elements_html_by_class,
    get_elements_text_and_html_by_attribute as get_elements_text_and_html_by_attribute,
    get_exe_version as get_exe_version,
    get_executable_path as get_executable_path,
    get_filesystem_encoding as get_filesystem_encoding,
    get_system_config_dirs as get_system_config_dirs,
    get_user_config_dirs as get_user_config_dirs,
    get_windows_version as get_windows_version,
    hyphenate_date as hyphenate_date,
    int_or_none as int_or_none,
    iri_to_uri as iri_to_uri,
    is_html as is_html,
    is_iterable_like as is_iterable_like,
    is_outdated_version as is_outdated_version,
    is_path_like as is_path_like,
    join_nonempty as join_nonempty,
    js_to_json as js_to_json,
    jwt_decode_hs256 as jwt_decode_hs256,
    jwt_encode as jwt_encode,
    limit_length as limit_length,
    locked_file as locked_file,
    long_to_bytes as long_to_bytes,
    lookup_unit_table as lookup_unit_table,
    lowercase_escape as lowercase_escape,
    make_archive_id as make_archive_id,
    make_dir as make_dir,
    match_filter_func as match_filter_func,
    match_str as match_str,
    merge_dicts as merge_dicts,
    merge_headers as merge_headers,
    mimetype2ext as mimetype2ext,
    month_by_abbreviation as month_by_abbreviation,
    month_by_name as month_by_name,
    multipart_encode as multipart_encode,
    netrc_from_content as netrc_from_content,
    number_of_digits as number_of_digits,
    ohdave_rsa_encrypt as ohdave_rsa_encrypt,
    orderedSet as orderedSet,
    orderedSet_from_options as orderedSet_from_options,
    parse_age_limit as parse_age_limit,
    parse_bitrate as parse_bitrate,
    parse_bytes as parse_bytes,
    parse_codecs as parse_codecs,
    parse_count as parse_count,
    parse_dfxp_time_expr as parse_dfxp_time_expr,
    parse_duration as parse_duration,
    parse_filesize as parse_filesize,
    parse_http_range as parse_http_range,
    parse_iso8601 as parse_iso8601,
    parse_list as parse_list,
    parse_m3u8_attributes as parse_m3u8_attributes,
    parse_qs as parse_qs,
    parse_resolution as parse_resolution,
    partial_application as partial_application,
    pkcs1pad as pkcs1pad,
    preferredencoding as preferredencoding,
    prepend_extension as prepend_extension,
    qualities as qualities,
    random_birthday as random_birthday,
    random_uuidv4 as random_uuidv4,
    read_batch_urls as read_batch_urls,
    read_stdin as read_stdin,
    remove_end as remove_end,
    remove_quotes as remove_quotes,
    remove_start as remove_start,
    remove_terminal_sequences as remove_terminal_sequences,
    render_table as render_table,
    replace_extension as replace_extension,
    rot47 as rot47,
    sanitize_filename as sanitize_filename,
    sanitize_open as sanitize_open,
    sanitize_path as sanitize_path,
    sanitize_url as sanitize_url,
    scale_thumbnails_to_max_format_width as scale_thumbnails_to_max_format_width,
    setproctitle as setproctitle,
    shell_quote as shell_quote,
    smuggle_url as smuggle_url,
    srt_subtitles_timecode as srt_subtitles_timecode,
    str_or_none as str_or_none,
    str_to_int as str_to_int,
    strftime_or_none as strftime_or_none,
    strip_jsonp as strip_jsonp,
    strip_or_none as strip_or_none,
    subtitles_filename as subtitles_filename,
    supports_terminal_sequences as supports_terminal_sequences,
    system_identifier as system_identifier,
    time_seconds as time_seconds,
    timeconvert as timeconvert,
    timetuple_from_msec as timetuple_from_msec,
    to_high_limit_path as to_high_limit_path,
    truncate_string as truncate_string,
    try_call as try_call,
    try_get as try_get,
    unescapeHTML as unescapeHTML,
    unified_strdate as unified_strdate,
    unified_timestamp as unified_timestamp,
    unsmuggle_url as unsmuggle_url,
    update_url as update_url,
    update_url_query as update_url_query,
    uppercase_escape as uppercase_escape,
    url_basename as url_basename,
    url_or_none as url_or_none,
    urlencode_postdata as urlencode_postdata,
    urlhandle_detect_ext as urlhandle_detect_ext,
    urljoin as urljoin,
    urshift as urshift,
    variadic as variadic,
    version_tuple as version_tuple,
    windows_enable_vt_mode as windows_enable_vt_mode,
    write_json_file as write_json_file,
    write_string as write_string,
    write_xattr as write_xattr,
    xpath_attr as xpath_attr,
    xpath_element as xpath_element,
    xpath_text as xpath_text,
    xpath_with_ns as xpath_with_ns,
    ytdl_is_updateable as ytdl_is_updateable,
)
from .traversal import (
    dict_get as dict_get,
    find_element as find_element,
    find_elements as find_elements,
    get_first as get_first,
    require as require,
    subs_list_to_dict as subs_list_to_dict,
    traverse_obj as traverse_obj,
    trim_str as trim_str,
    unpack as unpack,
    value as value,
)



from _typeshed import Incomplete

from pony.orm.core import Database, Entity

db: Database

class Person(Entity):
    __slots__ = ()
    id: Incomplete
    name: Incomplete
    dob: Incomplete
    ssn: Incomplete

class Student(Person):
    __slots__ = ()
    group: Incomplete
    mentor: Incomplete
    attend_courses: Incomplete

class Teacher(Person):
    __slots__ = ()
    teach_courses: Incomplete
    apprentices: Incomplete
    salary: Incomplete

class Assistant(Student, Teacher):
    __slots__ = ()

class Professor(Teacher):
    __slots__ = ()
    position: Incomplete

class Group(Entity):
    __slots__ = ()
    number: Incomplete
    students: Incomplete

class Course(Entity):
    __slots__ = ()
    name: Incomplete
    semester: Incomplete
    students: Incomplete
    teachers: Incomplete

def populate_database() -> None: ...
def show_all_persons() -> None: ...



import logging
import threading
from collections.abc import Mapping
from datetime import timedelta
from logging.config import _DictConfigArgs
from socket import SocketKind
from typing import Annotated, Any, ClassVar, Literal, TypedDict, type_check_only
from typing_extensions import TypeAlias

from gunicorn.http import Request
from gunicorn.http.wsgi import Response

from ._types import _EnvironType
from .config import Config

SYSLOG_FACILITIES: dict[str, int]

@type_check_only
class _AtomsDict(TypedDict, total=False):
    h: str
    l: str
    u: str
    t: str
    r: str
    s: str
    m: str | None
    U: str | None
    q: str | None
    H: str | None
    b: str
    B: int | None
    f: str
    a: str
    T: int
    D: int
    M: int
    L: str
    p: str

_CriticalIntType: TypeAlias = Annotated[int, "50"]
_ErrorIntType: TypeAlias = Annotated[int, "40"]
_WarningIntType: TypeAlias = Annotated[int, "30"]
_InfoIntType: TypeAlias = Annotated[int, "20"]
_DebugIntType: TypeAlias = Annotated[int, "10"]
_LogLevelIntType: TypeAlias = _CriticalIntType | _ErrorIntType | _WarningIntType | _InfoIntType | _DebugIntType
_LogLevelStrType: TypeAlias = Literal["critical", "error", "warning", "info", "debug"]
_LogLevelType: TypeAlias = _LogLevelIntType | _LogLevelStrType

CONFIG_DEFAULTS: _DictConfigArgs

def loggers() -> list[logging.Logger]: ...

class SafeAtoms(dict[str, Any]):
    def __init__(self, atoms: dict[str, Any]) -> None: ...
    def __getitem__(self, k: str) -> str: ...

_SyslogAddressType: TypeAlias = (
    tuple[Literal[SocketKind.SOCK_DGRAM] | None, str]  # Unix Socket
    | tuple[Literal[SocketKind.SOCK_DGRAM, SocketKind.SOCK_STREAM], tuple[str, int]]  # TCP/UDP Socket
)

def parse_syslog_address(addr: str) -> _SyslogAddressType: ...
@type_check_only
class _LogLevels(TypedDict):
    critical: _CriticalIntType
    error: _ErrorIntType
    warning: _WarningIntType
    info: _InfoIntType
    debug: _DebugIntType

class Logger:
    LOG_LEVELS: ClassVar[_LogLevels]
    loglevel: ClassVar[_LogLevelIntType]
    error_fmt: ClassVar[str]
    datefmt: ClassVar[str]
    access_fmt: ClassVar[str]
    syslog_fmt: ClassVar[str]
    atoms_wrapper_class: ClassVar[type[SafeAtoms]]
    error_log: logging.Logger
    access_log: logging.Logger
    error_handlers: list[logging.Handler]
    access_handlers: list[logging.Handler]
    logfile: Any | None
    lock: threading.Lock
    cfg: Config

    def __init__(self, cfg: Config) -> None: ...
    def setup(self, cfg: Config) -> None: ...
    def critical(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def error(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def warning(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def info(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def debug(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def exception(
        self,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = True,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def log(
        self,
        lvl: _LogLevelType,
        msg: object,
        *args: object,
        exc_info: logging._ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def atoms(self, resp: Response, req: Request, environ: _EnvironType, request_time: timedelta) -> _AtomsDict: ...
    def access(self, resp: Response, req: Request, environ: _EnvironType, request_time: timedelta) -> None: ...
    def now(self) -> str: ...
    def reopen_files(self) -> None: ...
    def close_on_exec(self) -> None: ...



import sys

from . import (
    bdist,
    bdist_dumb,
    bdist_rpm,
    build,
    build_clib,
    build_ext,
    build_py,
    build_scripts,
    check,
    clean,
    install,
    install_data,
    install_headers,
    install_lib,
    install_scripts,
    register,
    sdist,
    upload,
)

__all__ = [
    "build",
    "build_py",
    "build_ext",
    "build_clib",
    "build_scripts",
    "clean",
    "install",
    "install_lib",
    "install_headers",
    "install_scripts",
    "install_data",
    "sdist",
    "register",
    "bdist",
    "bdist_dumb",
    "bdist_rpm",
    "check",
    "upload",
]

if sys.version_info < (3, 10):
    from . import bdist_wininst

    __all__ += ["bdist_wininst"]



from _typeshed import Unused
from typing import Final

from Xlib.display import Display
from Xlib.protocol import rq
from Xlib.xobject import resource

extname: Final = "XTEST"
CurrentCursor: Final = 1

class GetVersion(rq.ReplyRequest): ...

def get_version(self: Display | resource.Resource, major: int, minor: int) -> GetVersion: ...

class CompareCursor(rq.ReplyRequest): ...

def compare_cursor(self: Display | resource.Resource, cursor: int) -> int: ...

class FakeInput(rq.Request): ...

def fake_input(
    self: Display | resource.Resource, event_type: int, detail: int = 0, time: int = 0, root: int = 0, x: int = 0, y: int = 0
) -> None: ...

class GrabControl(rq.Request): ...

def grab_control(self: Display | resource.Resource, impervious: bool) -> None: ...
def init(disp: Display, info: Unused) -> None: ...



from __future__ import annotations

from typing import Any, Literal, Union
from typing_extensions import assert_type

from webob.cachecontrol import CacheControl
from webob.request import BaseRequest
from webob.response import Response

req = BaseRequest({})
res = Response()
assert_type(req.cache_control, CacheControl[Literal["request"]])
assert_type(res.cache_control, CacheControl[Literal["response"]])

assert_type(CacheControl.parse(""), CacheControl[None])
assert_type(CacheControl.parse("", type="request"), CacheControl[Literal["request"]])
assert_type(CacheControl.parse("", type="response"), CacheControl[Literal["response"]])

req_cc = req.cache_control
res_cc = res.cache_control
shared_cc = CacheControl.parse("")
assert_type(req_cc, CacheControl[Literal["request"]])
assert_type(res_cc, CacheControl[Literal["response"]])
assert_type(shared_cc, CacheControl[None])
any_cc = CacheControl[Any]({}, None)

assert_type(req_cc.max_stale, Union[int, Literal["*"], None])
res_cc.max_stale  # type: ignore
shared_cc.max_stale  # type: ignore
assert_type(any_cc.max_stale, Union[int, Literal["*"], None])

assert_type(req_cc.min_fresh, Union[int, None])
res_cc.min_fresh  # type: ignore
shared_cc.min_fresh  # type: ignore
assert_type(any_cc.min_fresh, Union[int, None])

assert_type(req_cc.only_if_cached, bool)
res_cc.only_if_cached  # type: ignore
shared_cc.only_if_cached  # type: ignore
assert_type(any_cc.only_if_cached, bool)

req_cc.public  # type: ignore
assert_type(res_cc.public, bool)
shared_cc.public  # type: ignore
assert_type(any_cc.public, bool)

# NOTE: pyright gets confused about the `Literal["*"]` the types match
req_cc.private  # type: ignore
assert_type(res_cc.private, Union[str, Literal["*"], None])  # pyright: ignore
shared_cc.private  # type: ignore
assert_type(any_cc.private, Union[str, Literal["*"], None])  # pyright: ignore

assert_type(req_cc.no_cache, Union[str, Literal["*"], None])  # pyright: ignore
assert_type(res_cc.no_cache, Union[str, Literal["*"], None])  # pyright: ignore
assert_type(shared_cc.no_cache, Union[str, Literal["*"], None])  # pyright: ignore
assert_type(any_cc.no_cache, Union[str, Literal["*"], None])  # pyright: ignore

assert_type(req_cc.no_store, bool)
assert_type(res_cc.no_store, bool)
assert_type(shared_cc.no_store, bool)
assert_type(any_cc.no_store, bool)

assert_type(req_cc.no_transform, bool)
assert_type(res_cc.no_transform, bool)
assert_type(shared_cc.no_transform, bool)
assert_type(any_cc.no_transform, bool)

req_cc.must_revalidate  # type: ignore
assert_type(res_cc.must_revalidate, bool)
shared_cc.must_revalidate  # type: ignore
assert_type(any_cc.must_revalidate, bool)

req_cc.proxy_revalidate  # type: ignore
assert_type(res_cc.proxy_revalidate, bool)
shared_cc.proxy_revalidate  # type: ignore
assert_type(any_cc.proxy_revalidate, bool)

# NOTE: pyright gets confused about the `Literal[-1]` the types match
assert_type(req_cc.max_age, Union[int, Literal[-1], None])  # pyright: ignore
assert_type(res_cc.max_age, Union[int, Literal[-1], None])  # pyright: ignore
assert_type(shared_cc.max_age, Union[int, Literal[-1], None])  # pyright: ignore
assert_type(any_cc.max_age, Union[int, Literal[-1], None])  # pyright: ignore

req_cc.s_maxage  # type: ignore
assert_type(res_cc.s_maxage, Union[int, None])
shared_cc.s_maxage  # type: ignore
assert_type(any_cc.s_maxage, Union[int, None])

req_cc.s_max_age  # type: ignore
assert_type(res_cc.s_max_age, Union[int, None])
shared_cc.s_max_age  # type: ignore
assert_type(any_cc.s_max_age, Union[int, None])

req_cc.stale_while_revalidate  # type: ignore
assert_type(res_cc.stale_while_revalidate, Union[int, None])
shared_cc.stale_while_revalidate  # type: ignore
assert_type(any_cc.stale_while_revalidate, Union[int, None])

req_cc.stale_if_error  # type: ignore
assert_type(res_cc.stale_if_error, Union[int, None])
shared_cc.stale_if_error  # type: ignore
assert_type(any_cc.stale_if_error, Union[int, None])



from ._typing import ArrayLike, OptGeoArrayLike

__all__ = ["assert_geometries_equal"]

def assert_geometries_equal(
    x: OptGeoArrayLike,
    y: OptGeoArrayLike,
    tolerance: ArrayLike[float] = 1e-7,
    equal_none: bool = True,
    equal_nan: bool = True,
    normalize: bool = False,
    err_msg: str = "",
    verbose: bool = True,
) -> None: ...



from _typeshed import ConvertibleToFloat, ConvertibleToInt, Unused
from collections.abc import Callable, Iterator
from typing import ClassVar, Literal, TypeVar
from typing_extensions import Self

from openpyxl.descriptors import Strict
from openpyxl.descriptors.base import Alias, Bool, Float, Integer, String, _ConvertibleToBool
from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.styles.styleable import StyleableObject
from openpyxl.utils.bound_dictionary import BoundDictionary
from openpyxl.utils.cell import _RangeBoundariesTuple
from openpyxl.worksheet.worksheet import Worksheet
from openpyxl.xml.functions import Element

_DimKeyT = TypeVar("_DimKeyT", bound=str | int)
_DimT = TypeVar("_DimT", bound=Dimension)

class Dimension(Strict, StyleableObject):
    __fields__: ClassVar[tuple[str, ...]]

    index: Integer[Literal[False]]
    hidden: Bool[Literal[False]]
    outlineLevel: Integer[Literal[True]]
    outline_level: Alias
    collapsed: Bool[Literal[False]]
    style: Alias  # type: ignore[assignment]

    # Dimensions are only meant to be used on Worksheet objects
    parent: Worksheet

    def __init__(
        self,
        index: ConvertibleToInt,
        hidden: _ConvertibleToBool,
        outlineLevel: ConvertibleToInt | None,
        collapsed: _ConvertibleToBool,
        worksheet: Worksheet,
        visible: Unused = True,
        style=None,
    ) -> None: ...
    def __iter__(self) -> Iterator[tuple[str, str]]: ...
    def __copy__(self) -> Self: ...

class RowDimension(Dimension):
    r: Alias
    s: Alias
    ht: Float[Literal[True]]
    height: Alias
    thickBot: Bool[Literal[False]]
    thickTop: Bool[Literal[False]]
    def __init__(
        self,
        worksheet: Worksheet,
        index: int = 0,
        ht: ConvertibleToFloat | None = None,
        customHeight: Unused = None,
        s=None,
        customFormat: Unused = None,
        hidden: _ConvertibleToBool = None,
        outlineLevel: ConvertibleToInt | None = 0,
        outline_level: ConvertibleToInt | None = None,
        collapsed: _ConvertibleToBool = None,
        visible=None,
        height=None,
        r=None,
        spans: Unused = None,
        thickBot: _ConvertibleToBool = None,
        thickTop: _ConvertibleToBool = None,
        **kw: Unused,
    ) -> None: ...
    @property
    def customFormat(self) -> bool: ...
    @property
    def customHeight(self) -> bool: ...

class ColumnDimension(Dimension):
    width: Float[Literal[False]]
    bestFit: Bool[Literal[False]]
    auto_size: Alias
    index: String[Literal[False]]  # type: ignore[assignment]
    min: Integer[Literal[True]]
    max: Integer[Literal[True]]
    collapsed: Bool[Literal[False]]

    def __init__(
        self,
        worksheet: Worksheet,
        index: str = "A",
        width: ConvertibleToFloat = 13,
        bestFit: _ConvertibleToBool = False,
        hidden: _ConvertibleToBool = False,
        outlineLevel: ConvertibleToInt | None = 0,
        outline_level: ConvertibleToInt | None = None,
        collapsed: _ConvertibleToBool = False,
        style=None,
        min: ConvertibleToInt | None = None,
        max: ConvertibleToInt | None = None,
        customWidth: Unused = False,
        visible: bool | None = None,
        auto_size: _ConvertibleToBool | None = None,
    ) -> None: ...
    @property
    def customWidth(self) -> bool: ...
    def reindex(self) -> None: ...
    @property
    def range(self) -> str: ...
    def to_tree(self) -> Element | None: ...

class DimensionHolder(BoundDictionary[_DimKeyT, _DimT]):
    worksheet: Worksheet
    max_outline: int | None
    default_factory: Callable[[], _DimT] | None

    def __init__(
        self, worksheet: Worksheet, reference: str = "index", default_factory: Callable[[], _DimT] | None = None
    ) -> None: ...
    def group(self, start: _DimKeyT, end: _DimKeyT | None = None, outline_level: int = 1, hidden: bool = False) -> None: ...
    def to_tree(self) -> Element | None: ...

class SheetFormatProperties(Serialisable):
    tagname: ClassVar[str]
    baseColWidth: Integer[Literal[True]]
    defaultColWidth: Float[Literal[True]]
    defaultRowHeight: Float[Literal[False]]
    customHeight: Bool[Literal[True]]
    zeroHeight: Bool[Literal[True]]
    thickTop: Bool[Literal[True]]
    thickBottom: Bool[Literal[True]]
    outlineLevelRow: Integer[Literal[True]]
    outlineLevelCol: Integer[Literal[True]]
    def __init__(
        self,
        baseColWidth: ConvertibleToInt | None = 8,
        defaultColWidth: ConvertibleToFloat | None = None,
        defaultRowHeight: ConvertibleToFloat = 15,
        customHeight: _ConvertibleToBool | None = None,
        zeroHeight: _ConvertibleToBool | None = None,
        thickTop: _ConvertibleToBool | None = None,
        thickBottom: _ConvertibleToBool | None = None,
        outlineLevelRow: ConvertibleToInt | None = None,
        outlineLevelCol: ConvertibleToInt | None = None,
    ) -> None: ...

class SheetDimension(Serialisable):
    tagname: ClassVar[str]
    ref: String[Literal[False]]
    def __init__(self, ref: str) -> None: ...
    @property
    def boundaries(self) -> _RangeBoundariesTuple: ...



import re
from _typeshed import Incomplete
from collections.abc import Callable
from typing import Final

CODE_VERIFIER_PATTERN: Final[re.Pattern[str]]
CODE_CHALLENGE_PATTERN: Final[re.Pattern[str]]

def create_s256_code_challenge(code_verifier): ...
def compare_plain_code_challenge(code_verifier, code_challenge): ...
def compare_s256_code_challenge(code_verifier, code_challenge): ...

class CodeChallenge:
    DEFAULT_CODE_CHALLENGE_METHOD: str
    SUPPORTED_CODE_CHALLENGE_METHOD: list[str]
    CODE_CHALLENGE_METHODS: dict[str, Callable[[Incomplete, Incomplete], Incomplete]]
    required: bool
    def __init__(self, required: bool = True) -> None: ...
    def __call__(self, grant) -> None: ...
    def validate_code_challenge(self, grant, redirect_uri) -> None: ...
    def validate_code_verifier(self, grant, result) -> None: ...
    def get_authorization_code_challenge(self, authorization_code): ...
    def get_authorization_code_challenge_method(self, authorization_code): ...



annotation_value_types: tuple[type, ...]

def is_classmethod(func: object) -> bool: ...  # argument func is passing to getattr() function
def is_instance_method(parent_class: type, func_name: str, func: object) -> bool: ...



from hvac.api.secrets_engines.active_directory import ActiveDirectory as ActiveDirectory
from hvac.api.secrets_engines.aws import Aws as Aws
from hvac.api.secrets_engines.azure import Azure as Azure
from hvac.api.secrets_engines.database import Database as Database
from hvac.api.secrets_engines.gcp import Gcp as Gcp
from hvac.api.secrets_engines.identity import Identity as Identity
from hvac.api.secrets_engines.kv import Kv as Kv
from hvac.api.secrets_engines.kv_v1 import KvV1 as KvV1
from hvac.api.secrets_engines.kv_v2 import KvV2 as KvV2
from hvac.api.secrets_engines.ldap import Ldap as Ldap
from hvac.api.secrets_engines.pki import Pki as Pki
from hvac.api.secrets_engines.rabbitmq import RabbitMQ as RabbitMQ
from hvac.api.secrets_engines.ssh import Ssh as Ssh
from hvac.api.secrets_engines.transform import Transform as Transform
from hvac.api.secrets_engines.transit import Transit as Transit
from hvac.api.vault_api_base import VaultApiBase
from hvac.api.vault_api_category import VaultApiCategory

__all__ = (
    "Aws",
    "Azure",
    "Gcp",
    "ActiveDirectory",
    "Identity",
    "Kv",
    "KvV1",
    "KvV2",
    "Ldap",
    "Pki",
    "Transform",
    "Transit",
    "SecretsEngines",
    "Database",
    "RabbitMQ",
    "Ssh",
)

class SecretsEngines(VaultApiCategory):
    implemented_classes: list[type[VaultApiBase]]
    unimplemented_classes: list[str]



from _typeshed import Incomplete, SupportsGetItem
from collections.abc import Mapping
from typing import SupportsIndex

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["equitable_color"]

@_dispatchable
def is_coloring(G: Graph[_Node], coloring: SupportsGetItem[Incomplete, Incomplete]) -> bool: ...
@_dispatchable
def is_equitable(G: Graph[_Node], coloring: Mapping[Incomplete, Incomplete], num_colors: SupportsIndex | None = None) -> bool: ...
def make_C_from_F(F): ...
def make_N_from_L_C(L, C): ...
def make_H_from_C_N(C, N): ...
def change_color(u, X, Y, N, H, F, C, L): ...
def move_witnesses(src_color, dst_color, N, H, F, C, T_cal, L): ...
@_dispatchable
def pad_graph(G: Graph[_Node], num_colors): ...
def procedure_P(V_minus, V_plus, N, H, F, C, L, excluded_colors=None): ...
@_dispatchable
def equitable_color(G: Graph[_Node], num_colors): ...



from _typeshed import Incomplete
from collections.abc import Callable, Mapping
from typing import Generic

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["treewidth_min_degree", "treewidth_min_fill_in"]

@_dispatchable
def treewidth_min_degree(G: Graph[_Node]) -> tuple[int, Graph[frozenset[_Node]]]: ...
@_dispatchable
def treewidth_min_fill_in(G: Graph[_Node]) -> tuple[int, Graph[frozenset[_Node]]]: ...

class MinDegreeHeuristic(Generic[_Node]):
    count: Incomplete

    def __init__(self, graph: Graph[_Node]) -> None: ...
    def best_node(self, graph: Mapping[_Node, set[_Node]]) -> _Node | None: ...

def min_fill_in_heuristic(graph_dict: Mapping[_Node, set[_Node]]) -> _Node | None: ...
@_dispatchable
def treewidth_decomp(
    G: Graph[_Node], heuristic: Callable[[dict[_Node, set[_Node]]], _Node | None] = ...
) -> tuple[int, Graph[frozenset[_Node]]]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.protobuf.data_service_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class DispatcherConfig(google.protobuf.message.Message):
    """Configuration for a tf.data service DispatchServer.
    Next id: 13
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PORT_FIELD_NUMBER: builtins.int
    PROTOCOL_FIELD_NUMBER: builtins.int
    WORK_DIR_FIELD_NUMBER: builtins.int
    FAULT_TOLERANT_MODE_FIELD_NUMBER: builtins.int
    WORKER_ADDRESSES_FIELD_NUMBER: builtins.int
    DEPLOYMENT_MODE_FIELD_NUMBER: builtins.int
    JOB_GC_CHECK_INTERVAL_MS_FIELD_NUMBER: builtins.int
    JOB_GC_TIMEOUT_MS_FIELD_NUMBER: builtins.int
    GC_DYNAMIC_SHARDING_JOBS_FIELD_NUMBER: builtins.int
    CLIENT_TIMEOUT_MS_FIELD_NUMBER: builtins.int
    WORKER_TIMEOUT_MS_FIELD_NUMBER: builtins.int
    WORKER_MAX_CONCURRENT_SNAPSHOTS_FIELD_NUMBER: builtins.int
    port: builtins.int
    """The port for the dispatcher to bind to. A value of 0 indicates that the
    dispatcher may bind to any available port.
    """
    protocol: builtins.str
    """The protocol for the dispatcher to use when connecting to workers."""
    work_dir: builtins.str
    """A work directory to use for storing dispatcher state, and for recovering
    during restarts. The empty string indicates not to use any work directory.
    """
    fault_tolerant_mode: builtins.bool
    """Whether to run in fault tolerant mode, where dispatcher state is saved
    across restarts. Requires that `work_dir` is nonempty.
    """
    deployment_mode: tensorflow.core.protobuf.data_service_pb2.DeploymentMode.ValueType
    """(Optional.) tf.data service deployment mode. Supported values are "REMOTE",
    "COLOCATED", and "HYBRID". If unspecified, it is assumed to be "REMOTE".
    """
    job_gc_check_interval_ms: builtins.int
    """How often the dispatcher should scan through to delete old and unused
    jobs. A value of 0 indicates that the decision should be left up to the
    runtime.
    """
    job_gc_timeout_ms: builtins.int
    """How long a job needs to be unused before it becomes a candidate for garbage
    collection. A value of -1 indicates that jobs should never be garbage
    collected. A value of 0 indicates that the decision should be left up to
    the runtime. Note: This does not apply to dynamic sharding unless users
    explicitly opt-in by enabling `gc_dynamic_sharding_jobs` below.
    """
    gc_dynamic_sharding_jobs: builtins.bool
    """Whether dynamically sharded jobs should be eligible for garbage collection.
    These jobs are not garbage collected by default, since if a job is garbage
    collected and then re-created, it will revisit all data from the start. If
    revisiting data is acceptible and you want automatic reclamation of
    iterator memory, set `gc_dynamic_sharding_jobs` to `true`.
    """
    client_timeout_ms: builtins.int
    """How long to wait before garbage-collecting a client that hasn't
    heartbeated to the dispatcher. A value of 0 indicates that the timeout
    should be left to the runtime.
    """
    worker_timeout_ms: builtins.int
    """How long to wait for a worker to heartbeat before considering it missing.
    A value of 0 indicates that the timeout should be left to the runtime.
    """
    worker_max_concurrent_snapshots: builtins.int
    """The maximum number of snapshots that a worker can concurrently process at a
    given point in time. This is a tradeoff between worker resource usage and
    snapshot wall time. A value of 0 indicates that the decision should be left
    up to the runtime.
    """
    @property
    def worker_addresses(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """(Optional.) If the job uses auto-sharding, it needs to specify a fixed list
        of worker addresses that will register with the dispatcher. The worker
        addresses should be in the format "host" or "host:port", where "port" is an
        integer, named port, or %port% to match any port.
        """

    def __init__(
        self,
        *,
        port: builtins.int | None = ...,
        protocol: builtins.str | None = ...,
        work_dir: builtins.str | None = ...,
        fault_tolerant_mode: builtins.bool | None = ...,
        worker_addresses: collections.abc.Iterable[builtins.str] | None = ...,
        deployment_mode: tensorflow.core.protobuf.data_service_pb2.DeploymentMode.ValueType | None = ...,
        job_gc_check_interval_ms: builtins.int | None = ...,
        job_gc_timeout_ms: builtins.int | None = ...,
        gc_dynamic_sharding_jobs: builtins.bool | None = ...,
        client_timeout_ms: builtins.int | None = ...,
        worker_timeout_ms: builtins.int | None = ...,
        worker_max_concurrent_snapshots: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "client_timeout_ms",
            b"client_timeout_ms",
            "deployment_mode",
            b"deployment_mode",
            "fault_tolerant_mode",
            b"fault_tolerant_mode",
            "gc_dynamic_sharding_jobs",
            b"gc_dynamic_sharding_jobs",
            "job_gc_check_interval_ms",
            b"job_gc_check_interval_ms",
            "job_gc_timeout_ms",
            b"job_gc_timeout_ms",
            "port",
            b"port",
            "protocol",
            b"protocol",
            "work_dir",
            b"work_dir",
            "worker_addresses",
            b"worker_addresses",
            "worker_max_concurrent_snapshots",
            b"worker_max_concurrent_snapshots",
            "worker_timeout_ms",
            b"worker_timeout_ms",
        ],
    ) -> None: ...

global___DispatcherConfig = DispatcherConfig

@typing.final
class WorkerConfig(google.protobuf.message.Message):
    """Configuration for a tf.data service WorkerServer.
    Next id: 14
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PORT_FIELD_NUMBER: builtins.int
    PROTOCOL_FIELD_NUMBER: builtins.int
    DISPATCHER_ADDRESS_FIELD_NUMBER: builtins.int
    WORKER_ADDRESS_FIELD_NUMBER: builtins.int
    WORKER_TAGS_FIELD_NUMBER: builtins.int
    HEARTBEAT_INTERVAL_MS_FIELD_NUMBER: builtins.int
    DISPATCHER_TIMEOUT_MS_FIELD_NUMBER: builtins.int
    DATA_TRANSFER_PROTOCOL_FIELD_NUMBER: builtins.int
    DATA_TRANSFER_PORT_FIELD_NUMBER: builtins.int
    DATA_TRANSFER_ADDRESS_FIELD_NUMBER: builtins.int
    CROSS_TRAINER_CACHE_SIZE_BYTES_FIELD_NUMBER: builtins.int
    SNAPSHOT_MAX_CHUNK_SIZE_BYTES_FIELD_NUMBER: builtins.int
    SHUTDOWN_QUIET_PERIOD_MS_FIELD_NUMBER: builtins.int
    port: builtins.int
    """The port for the worker to bind to. A value of 0 indicates that the
    worker may bind to any available port.
    """
    protocol: builtins.str
    """The protocol for the worker to use when connecting to the dispatcher."""
    dispatcher_address: builtins.str
    """The address of the dispatcher to register with."""
    worker_address: builtins.str
    """The address of the worker server. The substring "%port%", if specified,
    will be replaced with the worker's bound port. This is useful when the port
    is set to `0`.
    """
    heartbeat_interval_ms: builtins.int
    """How often the worker should heartbeat to the master. A value of 0 indicates
    that the decision should be left up to the runtime.
    """
    dispatcher_timeout_ms: builtins.int
    """How long to retry requests to the dispatcher before giving up and reporting
    an error. A value of 0 indicates that the decision should be left up to the
    runtime.
    """
    data_transfer_protocol: builtins.str
    """If set, the name of an alternative data transfer protocol for which the
    worker starts an additional server ("data transfer server"); the trainer
    can then get data from this server. If not set, no such server is started,
    and the trainer can only get data from the regular worker server over
    `protocol`.
    """
    data_transfer_port: builtins.int
    """If `data_transfer_protocol` is set, the port to which the data transfer
    server binds. If set to `0`, the server binds to any available port.
    """
    data_transfer_address: builtins.str
    """If `data_transfer_protocol` is set, the address of the data transfer
    server. The substring "%dts_port%" can be used to represent -- and is
    replaced with -- the bound port of the data transfer server; this is useful
    when `data_transfer_port` is set to `0`.
    """
    cross_trainer_cache_size_bytes: builtins.int
    """Maximum size of the cross-trainer cache in bytes. If enabled, make sure
    your training job provides sufficient memory resources.
    """
    snapshot_max_chunk_size_bytes: builtins.int
    """The maximum size of a distributed snapshot chunk file. A value of 0
    indicates that the decision should be left up to the runtime.
    """
    shutdown_quiet_period_ms: builtins.int
    """When shutting down a worker, how long to wait for the gRPC server to
    process the final requests. This is used to achieve clean shutdown in unit
    tests.
    """
    @property
    def worker_tags(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Tags attached to the worker. This allows reading from selected workers.
        For example, by applying a "COLOCATED" tag, tf.data service is able to read
        from the local tf.data worker if one exists, then from off-TF-host workers,
        to avoid cross-TF-host reads.
        """

    def __init__(
        self,
        *,
        port: builtins.int | None = ...,
        protocol: builtins.str | None = ...,
        dispatcher_address: builtins.str | None = ...,
        worker_address: builtins.str | None = ...,
        worker_tags: collections.abc.Iterable[builtins.str] | None = ...,
        heartbeat_interval_ms: builtins.int | None = ...,
        dispatcher_timeout_ms: builtins.int | None = ...,
        data_transfer_protocol: builtins.str | None = ...,
        data_transfer_port: builtins.int | None = ...,
        data_transfer_address: builtins.str | None = ...,
        cross_trainer_cache_size_bytes: builtins.int | None = ...,
        snapshot_max_chunk_size_bytes: builtins.int | None = ...,
        shutdown_quiet_period_ms: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "cross_trainer_cache_size_bytes",
            b"cross_trainer_cache_size_bytes",
            "data_transfer_address",
            b"data_transfer_address",
            "data_transfer_port",
            b"data_transfer_port",
            "data_transfer_protocol",
            b"data_transfer_protocol",
            "dispatcher_address",
            b"dispatcher_address",
            "dispatcher_timeout_ms",
            b"dispatcher_timeout_ms",
            "heartbeat_interval_ms",
            b"heartbeat_interval_ms",
            "port",
            b"port",
            "protocol",
            b"protocol",
            "shutdown_quiet_period_ms",
            b"shutdown_quiet_period_ms",
            "snapshot_max_chunk_size_bytes",
            b"snapshot_max_chunk_size_bytes",
            "worker_address",
            b"worker_address",
            "worker_tags",
            b"worker_tags",
        ],
    ) -> None: ...

global___WorkerConfig = WorkerConfig



from typing import Final

REMOTE_NAMESPACE: Final = "remote"
LOCAL_NAMESPACE: Final = "local"
LOCAL_EXCEPTIONS: tuple[type[Exception], ...]

async def begin_subsegment(session, trace_config_ctx, params): ...
async def end_subsegment(session, trace_config_ctx, params): ...
async def end_subsegment_with_exception(session, trace_config_ctx, params): ...
def aws_xray_trace_config(name=None): ...



from typing import Any, Literal

from docker.types import IPAMConfig

from .containers import Container
from .resource import Collection, Model

class Network(Model):
    @property
    def name(self) -> str | None: ...
    @property
    def containers(self) -> list[Container]: ...
    def connect(self, container: str | Container, *args, **kwargs) -> None: ...
    def disconnect(self, container: str | Container, force: bool = False) -> None: ...
    def remove(self) -> None: ...

class NetworkCollection(Collection[Network]):
    model: type[Network]
    def create(  # type:ignore[override]
        self,
        name: str,
        driver: str | None = None,
        options: dict[str, Any] | None = None,
        ipam: IPAMConfig | None = None,
        check_duplicate: bool | None = None,
        internal: bool = False,
        labels: dict[str, Any] | None = None,
        enable_ipv6: bool = False,
        attachable: bool | None = None,
        scope: Literal["local", "global", "swarm"] | None = None,
        ingress: bool | None = None,
    ) -> Network: ...
    def get(
        self, network_id: str, verbose: bool | None = None, scope: Literal["local", "global", "swarm"] | None = None
    ) -> Network: ...  # type:ignore[override]
    def list(self, *args, **kwargs) -> list[Network]: ...
    def prune(self, filters: dict[str, Any] | None = None) -> dict[str, Any]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protocol Buffers - Google's data interchange format
Copyright 2008 Google Inc.  All rights reserved.
https://developers.google.com/protocol-buffers/

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above
copyright notice, this list of conditions and the following disclaimer
in the documentation and/or other materials provided with the
distribution.
    * Neither the name of Google Inc. nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.internal.well_known_types
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Duration(google.protobuf.message.Message, google.protobuf.internal.well_known_types.Duration):
    """A Duration represents a signed, fixed-length span of time represented
    as a count of seconds and fractions of seconds at nanosecond
    resolution. It is independent of any calendar and concepts like "day"
    or "month". It is related to Timestamp in that the difference between
    two Timestamp values is a Duration and it can be added or subtracted
    from a Timestamp. Range is approximately +-10,000 years.

    # Examples

    Example 1: Compute Duration from two Timestamps in pseudo code.

        Timestamp start = ...;
        Timestamp end = ...;
        Duration duration = ...;

        duration.seconds = end.seconds - start.seconds;
        duration.nanos = end.nanos - start.nanos;

        if (duration.seconds < 0 && duration.nanos > 0) {
          duration.seconds += 1;
          duration.nanos -= 1000000000;
        } else if (duration.seconds > 0 && duration.nanos < 0) {
          duration.seconds -= 1;
          duration.nanos += 1000000000;
        }

    Example 2: Compute Timestamp from Timestamp + Duration in pseudo code.

        Timestamp start = ...;
        Duration duration = ...;
        Timestamp end = ...;

        end.seconds = start.seconds + duration.seconds;
        end.nanos = start.nanos + duration.nanos;

        if (end.nanos < 0) {
          end.seconds -= 1;
          end.nanos += 1000000000;
        } else if (end.nanos >= 1000000000) {
          end.seconds += 1;
          end.nanos -= 1000000000;
        }

    Example 3: Compute Duration from datetime.timedelta in Python.

        td = datetime.timedelta(days=3, minutes=10)
        duration = Duration()
        duration.FromTimedelta(td)

    # JSON Mapping

    In JSON format, the Duration type is encoded as a string rather than an
    object, where the string ends in the suffix "s" (indicating seconds) and
    is preceded by the number of seconds, with nanoseconds expressed as
    fractional seconds. For example, 3 seconds with 0 nanoseconds should be
    encoded in JSON format as "3s", while 3 seconds and 1 nanosecond should
    be expressed in JSON format as "3.000000001s", and 3 seconds and 1
    microsecond should be expressed in JSON format as "3.000001s".
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECONDS_FIELD_NUMBER: builtins.int
    NANOS_FIELD_NUMBER: builtins.int
    seconds: builtins.int
    """Signed seconds of the span of time. Must be from -315,576,000,000
    to +315,576,000,000 inclusive. Note: these bounds are computed from:
    60 sec/min * 60 min/hr * 24 hr/day * 365.25 days/year * 10000 years
    """
    nanos: builtins.int
    """Signed fractions of a second at nanosecond resolution of the span
    of time. Durations less than one second are represented with a 0
    `seconds` field and a positive or negative `nanos` field. For durations
    of one second or more, a non-zero value for the `nanos` field must be
    of the same sign as the `seconds` field. Must be from -999,999,999
    to +999,999,999 inclusive.
    """
    def __init__(self, *, seconds: builtins.int | None = ..., nanos: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["nanos", b"nanos", "seconds", b"seconds"]) -> None: ...

global___Duration = Duration



import sys
from _typeshed import MaybeNone
from collections.abc import Callable, Iterable, Iterator
from types import GenericAlias
from typing import Any, Generic, Literal, SupportsComplex, SupportsFloat, SupportsIndex, SupportsInt, TypeVar, overload
from typing_extensions import Self, TypeAlias, disjoint_base

_T = TypeVar("_T")
_S = TypeVar("_S")
_N = TypeVar("_N", int, float, SupportsFloat, SupportsInt, SupportsIndex, SupportsComplex)
_T_co = TypeVar("_T_co", covariant=True)
_S_co = TypeVar("_S_co", covariant=True)
_T1 = TypeVar("_T1")
_T2 = TypeVar("_T2")
_T3 = TypeVar("_T3")
_T4 = TypeVar("_T4")
_T5 = TypeVar("_T5")
_T6 = TypeVar("_T6")
_T7 = TypeVar("_T7")
_T8 = TypeVar("_T8")
_T9 = TypeVar("_T9")
_T10 = TypeVar("_T10")

_Step: TypeAlias = SupportsFloat | SupportsInt | SupportsIndex | SupportsComplex

_Predicate: TypeAlias = Callable[[_T], object]

# Technically count can take anything that implements a number protocol and has an add method
# but we can't enforce the add method
@disjoint_base
class count(Generic[_N]):
    @overload
    def __new__(cls) -> count[int]: ...
    @overload
    def __new__(cls, start: _N, step: _Step = ...) -> count[_N]: ...
    @overload
    def __new__(cls, *, step: _N) -> count[_N]: ...
    def __next__(self) -> _N: ...
    def __iter__(self) -> Self: ...

@disjoint_base
class cycle(Generic[_T]):
    def __new__(cls, iterable: Iterable[_T], /) -> Self: ...
    def __next__(self) -> _T: ...
    def __iter__(self) -> Self: ...

@disjoint_base
class repeat(Generic[_T]):
    @overload
    def __new__(cls, object: _T) -> Self: ...
    @overload
    def __new__(cls, object: _T, times: int) -> Self: ...
    def __next__(self) -> _T: ...
    def __iter__(self) -> Self: ...
    def __length_hint__(self) -> int: ...

@disjoint_base
class accumulate(Generic[_T]):
    @overload
    def __new__(cls, iterable: Iterable[_T], func: None = None, *, initial: _T | None = ...) -> Self: ...
    @overload
    def __new__(cls, iterable: Iterable[_S], func: Callable[[_T, _S], _T], *, initial: _T | None = ...) -> Self: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T: ...

@disjoint_base
class chain(Generic[_T]):
    def __new__(cls, *iterables: Iterable[_T]) -> Self: ...
    def __next__(self) -> _T: ...
    def __iter__(self) -> Self: ...
    @classmethod
    # We use type[Any] and not type[_S] to not lose the type inference from __iterable
    def from_iterable(cls: type[Any], iterable: Iterable[Iterable[_S]], /) -> chain[_S]: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias: ...

@disjoint_base
class compress(Generic[_T]):
    def __new__(cls, data: Iterable[_T], selectors: Iterable[Any]) -> Self: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T: ...

@disjoint_base
class dropwhile(Generic[_T]):
    def __new__(cls, predicate: _Predicate[_T], iterable: Iterable[_T], /) -> Self: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T: ...

@disjoint_base
class filterfalse(Generic[_T]):
    def __new__(cls, function: _Predicate[_T] | None, iterable: Iterable[_T], /) -> Self: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T: ...

@disjoint_base
class groupby(Generic[_T_co, _S_co]):
    @overload
    def __new__(cls, iterable: Iterable[_T1], key: None = None) -> groupby[_T1, _T1]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T1], key: Callable[[_T1], _T2]) -> groupby[_T2, _T1]: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> tuple[_T_co, Iterator[_S_co]]: ...

@disjoint_base
class islice(Generic[_T]):
    @overload
    def __new__(cls, iterable: Iterable[_T], stop: int | None, /) -> Self: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], start: int | None, stop: int | None, step: int | None = ..., /) -> Self: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T: ...

@disjoint_base
class starmap(Generic[_T_co]):
    def __new__(cls, function: Callable[..., _T], iterable: Iterable[Iterable[Any]], /) -> starmap[_T]: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T_co: ...

@disjoint_base
class takewhile(Generic[_T]):
    def __new__(cls, predicate: _Predicate[_T], iterable: Iterable[_T], /) -> Self: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T: ...

def tee(iterable: Iterable[_T], n: int = 2, /) -> tuple[Iterator[_T], ...]: ...
@disjoint_base
class zip_longest(Generic[_T_co]):
    # one iterable (fillvalue doesn't matter)
    @overload
    def __new__(cls, iter1: Iterable[_T1], /, *, fillvalue: object = ...) -> zip_longest[tuple[_T1]]: ...
    # two iterables
    @overload
    # In the overloads without fillvalue, all of the tuple members could theoretically be None,
    # but we return Any instead to avoid false positives for code where we know one of the iterables
    # is longer.
    def __new__(cls, iter1: Iterable[_T1], iter2: Iterable[_T2], /) -> zip_longest[tuple[_T1 | MaybeNone, _T2 | MaybeNone]]: ...
    @overload
    def __new__(
        cls, iter1: Iterable[_T1], iter2: Iterable[_T2], /, *, fillvalue: _T
    ) -> zip_longest[tuple[_T1 | _T, _T2 | _T]]: ...
    # three iterables
    @overload
    def __new__(
        cls, iter1: Iterable[_T1], iter2: Iterable[_T2], iter3: Iterable[_T3], /
    ) -> zip_longest[tuple[_T1 | MaybeNone, _T2 | MaybeNone, _T3 | MaybeNone]]: ...
    @overload
    def __new__(
        cls, iter1: Iterable[_T1], iter2: Iterable[_T2], iter3: Iterable[_T3], /, *, fillvalue: _T
    ) -> zip_longest[tuple[_T1 | _T, _T2 | _T, _T3 | _T]]: ...
    # four iterables
    @overload
    def __new__(
        cls, iter1: Iterable[_T1], iter2: Iterable[_T2], iter3: Iterable[_T3], iter4: Iterable[_T4], /
    ) -> zip_longest[tuple[_T1 | MaybeNone, _T2 | MaybeNone, _T3 | MaybeNone, _T4 | MaybeNone]]: ...
    @overload
    def __new__(
        cls, iter1: Iterable[_T1], iter2: Iterable[_T2], iter3: Iterable[_T3], iter4: Iterable[_T4], /, *, fillvalue: _T
    ) -> zip_longest[tuple[_T1 | _T, _T2 | _T, _T3 | _T, _T4 | _T]]: ...
    # five iterables
    @overload
    def __new__(
        cls, iter1: Iterable[_T1], iter2: Iterable[_T2], iter3: Iterable[_T3], iter4: Iterable[_T4], iter5: Iterable[_T5], /
    ) -> zip_longest[tuple[_T1 | MaybeNone, _T2 | MaybeNone, _T3 | MaybeNone, _T4 | MaybeNone, _T5 | MaybeNone]]: ...
    @overload
    def __new__(
        cls,
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        iter4: Iterable[_T4],
        iter5: Iterable[_T5],
        /,
        *,
        fillvalue: _T,
    ) -> zip_longest[tuple[_T1 | _T, _T2 | _T, _T3 | _T, _T4 | _T, _T5 | _T]]: ...
    # six or more iterables
    @overload
    def __new__(
        cls,
        iter1: Iterable[_T],
        iter2: Iterable[_T],
        iter3: Iterable[_T],
        iter4: Iterable[_T],
        iter5: Iterable[_T],
        iter6: Iterable[_T],
        /,
        *iterables: Iterable[_T],
    ) -> zip_longest[tuple[_T | MaybeNone, ...]]: ...
    @overload
    def __new__(
        cls,
        iter1: Iterable[_T],
        iter2: Iterable[_T],
        iter3: Iterable[_T],
        iter4: Iterable[_T],
        iter5: Iterable[_T],
        iter6: Iterable[_T],
        /,
        *iterables: Iterable[_T],
        fillvalue: _T,
    ) -> zip_longest[tuple[_T, ...]]: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T_co: ...

@disjoint_base
class product(Generic[_T_co]):
    @overload
    def __new__(cls, iter1: Iterable[_T1], /) -> product[tuple[_T1]]: ...
    @overload
    def __new__(cls, iter1: Iterable[_T1], iter2: Iterable[_T2], /) -> product[tuple[_T1, _T2]]: ...
    @overload
    def __new__(cls, iter1: Iterable[_T1], iter2: Iterable[_T2], iter3: Iterable[_T3], /) -> product[tuple[_T1, _T2, _T3]]: ...
    @overload
    def __new__(
        cls, iter1: Iterable[_T1], iter2: Iterable[_T2], iter3: Iterable[_T3], iter4: Iterable[_T4], /
    ) -> product[tuple[_T1, _T2, _T3, _T4]]: ...
    @overload
    def __new__(
        cls, iter1: Iterable[_T1], iter2: Iterable[_T2], iter3: Iterable[_T3], iter4: Iterable[_T4], iter5: Iterable[_T5], /
    ) -> product[tuple[_T1, _T2, _T3, _T4, _T5]]: ...
    @overload
    def __new__(
        cls,
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        iter4: Iterable[_T4],
        iter5: Iterable[_T5],
        iter6: Iterable[_T6],
        /,
    ) -> product[tuple[_T1, _T2, _T3, _T4, _T5, _T6]]: ...
    @overload
    def __new__(
        cls,
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        iter4: Iterable[_T4],
        iter5: Iterable[_T5],
        iter6: Iterable[_T6],
        iter7: Iterable[_T7],
        /,
    ) -> product[tuple[_T1, _T2, _T3, _T4, _T5, _T6, _T7]]: ...
    @overload
    def __new__(
        cls,
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        iter4: Iterable[_T4],
        iter5: Iterable[_T5],
        iter6: Iterable[_T6],
        iter7: Iterable[_T7],
        iter8: Iterable[_T8],
        /,
    ) -> product[tuple[_T1, _T2, _T3, _T4, _T5, _T6, _T7, _T8]]: ...
    @overload
    def __new__(
        cls,
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        iter4: Iterable[_T4],
        iter5: Iterable[_T5],
        iter6: Iterable[_T6],
        iter7: Iterable[_T7],
        iter8: Iterable[_T8],
        iter9: Iterable[_T9],
        /,
    ) -> product[tuple[_T1, _T2, _T3, _T4, _T5, _T6, _T7, _T8, _T9]]: ...
    @overload
    def __new__(
        cls,
        iter1: Iterable[_T1],
        iter2: Iterable[_T2],
        iter3: Iterable[_T3],
        iter4: Iterable[_T4],
        iter5: Iterable[_T5],
        iter6: Iterable[_T6],
        iter7: Iterable[_T7],
        iter8: Iterable[_T8],
        iter9: Iterable[_T9],
        iter10: Iterable[_T10],
        /,
    ) -> product[tuple[_T1, _T2, _T3, _T4, _T5, _T6, _T7, _T8, _T9, _T10]]: ...
    @overload
    def __new__(cls, *iterables: Iterable[_T1], repeat: int = 1) -> product[tuple[_T1, ...]]: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T_co: ...

@disjoint_base
class permutations(Generic[_T_co]):
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[2]) -> permutations[tuple[_T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[3]) -> permutations[tuple[_T, _T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[4]) -> permutations[tuple[_T, _T, _T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[5]) -> permutations[tuple[_T, _T, _T, _T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: int | None = ...) -> permutations[tuple[_T, ...]]: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T_co: ...

@disjoint_base
class combinations(Generic[_T_co]):
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[2]) -> combinations[tuple[_T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[3]) -> combinations[tuple[_T, _T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[4]) -> combinations[tuple[_T, _T, _T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[5]) -> combinations[tuple[_T, _T, _T, _T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: int) -> combinations[tuple[_T, ...]]: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T_co: ...

@disjoint_base
class combinations_with_replacement(Generic[_T_co]):
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[2]) -> combinations_with_replacement[tuple[_T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[3]) -> combinations_with_replacement[tuple[_T, _T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[4]) -> combinations_with_replacement[tuple[_T, _T, _T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: Literal[5]) -> combinations_with_replacement[tuple[_T, _T, _T, _T, _T]]: ...
    @overload
    def __new__(cls, iterable: Iterable[_T], r: int) -> combinations_with_replacement[tuple[_T, ...]]: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> _T_co: ...

if sys.version_info >= (3, 10):
    @disjoint_base
    class pairwise(Generic[_T_co]):
        def __new__(cls, iterable: Iterable[_T], /) -> pairwise[tuple[_T, _T]]: ...
        def __iter__(self) -> Self: ...
        def __next__(self) -> _T_co: ...

if sys.version_info >= (3, 12):
    @disjoint_base
    class batched(Generic[_T_co]):
        if sys.version_info >= (3, 13):
            def __new__(cls, iterable: Iterable[_T_co], n: int, *, strict: bool = False) -> Self: ...
        else:
            def __new__(cls, iterable: Iterable[_T_co], n: int) -> Self: ...

        def __iter__(self) -> Self: ...
        def __next__(self) -> tuple[_T_co, ...]: ...



from pathlib import Path
from typing import ClassVar, Final

from docutils.writers import _html_base

__docformat__: Final = "reStructuredText"

class Writer(_html_base.Writer):
    default_stylesheets: ClassVar[list[str]]
    default_stylesheet_dirs: ClassVar[list[str]]
    default_template: ClassVar[Path]
    translator_class: type[HTMLTranslator]

class HTMLTranslator(_html_base.HTMLTranslator):
    supported_block_tags: set[str]
    supported_inline_tags: set[str]



from _typeshed import Incomplete, SupportsKeysAndGetItem
from collections.abc import Callable, Iterable, Mapping
from html.parser import HTMLParser
from logging import Logger
from typing import ClassVar, Final, Literal
from typing_extensions import TypeAlias

from fpdf import FPDF

from .enums import Align, TextEmphasis
from .fonts import FontFace
from .table import Row, Table

__author__: Final[str]
__copyright__: Final[str]

_OLType: TypeAlias = Literal["1", "a", "A", "I", "i"]

LOGGER: Logger
MESSAGE_WAITING_WIN1252: Final = "\x95"
BULLET_UNICODE: Final = "•"
DEGREE_SIGN_WIN1252: Final = "\xb0"
RING_OPERATOR_UNICODE: Final = "∘"
HEADING_TAGS: Final[tuple[str, ...]]
DEFAULT_TAG_STYLES: Final[dict[str, FontFace]]
INLINE_TAGS: Final[tuple[str, ...]]
BLOCK_TAGS: Final[tuple[str, ...]]

COLOR_DICT: Final[dict[str, str]]

def color_as_decimal(color: str | None = "#000000") -> tuple[int, int, int] | None: ...
def parse_css_style(style_attr: str) -> dict[str, str]: ...

class HTML2FPDF(HTMLParser):
    HTML_UNCLOSED_TAGS: ClassVar[tuple[str, ...]]
    TABLE_LINE_HEIGHT: ClassVar[float]

    pdf: FPDF
    image_map: Callable[[str], str]
    ul_bullet_char: str
    li_prefix_color: tuple[int, int, int]
    warn_on_tags_not_matching: bool

    font_family: str
    font_size_pt: float
    font_emphasis: TextEmphasis
    font_color: tuple[int, int, int]

    style_stack: list[FontFace]
    h: float
    follows_trailing_space: bool
    follows_heading: bool
    href: str
    align: float | Align | None
    indent: int
    line_height_stack: list[Incomplete]
    ol_type: dict[int, _OLType]
    bullet: list[Incomplete]
    heading_level: Incomplete | None
    render_title_tag: bool
    table_line_separators: bool
    table: Table | None
    table_row: Row | None
    tr: dict[str, str] | None
    td_th: dict[str, str] | None
    tag_indents: dict[str, int]
    tag_styles: dict[str, FontFace]

    def __init__(
        self,
        pdf: FPDF,
        image_map: Callable[[str], str] | None = None,
        li_tag_indent: int | None = None,
        dd_tag_indent: int | None = None,
        table_line_separators: bool = False,
        ul_bullet_char: str = "disc",
        li_prefix_color: tuple[int, int, int] = (190, 0, 0),
        heading_sizes: SupportsKeysAndGetItem[str, int] | Iterable[tuple[str, int]] | None = None,
        pre_code_font: str | None = None,
        warn_on_tags_not_matching: bool = True,
        tag_indents: dict[str, int] | None = None,
        tag_styles: Mapping[str, FontFace] | None = None,
        font_family: str = "times",
        render_title_tag: bool = False,
    ) -> None: ...
    def handle_data(self, data) -> None: ...
    def handle_starttag(self, tag, attrs) -> None: ...
    def handle_endtag(self, tag) -> None: ...
    def put_link(self, text) -> None: ...
    def render_toc(self, pdf, outline) -> None: ...
    def error(self, message: str) -> None: ...

def ul_prefix(ul_type: str, is_ttf_font: bool | None) -> str: ...
def ol_prefix(ol_type: _OLType, index: int) -> str: ...

class HTMLMixin:
    def __init__(self, *args, **kwargs) -> None: ...



from _typeshed import Incomplete, SupportsGetItem
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["maximum_matching", "hopcroft_karp_matching", "eppstein_matching", "to_vertex_cover", "minimum_weight_full_matching"]

@_dispatchable
def hopcroft_karp_matching(G: Graph[_Node], top_nodes: Iterable[_Node] | None = None) -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def eppstein_matching(G: Graph[_Node], top_nodes: Iterable[Incomplete] | None = None) -> dict[Incomplete, Incomplete]: ...
@_dispatchable
def to_vertex_cover(
    G: Graph[_Node], matching: SupportsGetItem[Incomplete, Incomplete], top_nodes: Iterable[Incomplete] | None = None
): ...

maximum_matching = hopcroft_karp_matching

@_dispatchable
def minimum_weight_full_matching(
    G: Graph[_Node], top_nodes: Iterable[Incomplete] | None = None, weight: str | None = "weight"
) -> dict[Incomplete, Incomplete]: ...



from _typeshed import ReadableBuffer
from collections.abc import Callable, Generator, Iterable, Sized
from html.parser import HTMLParser
from re import Pattern
from typing import Literal, overload

def camel2under(camel_string: str) -> str: ...
def under2camel(under_string: str) -> str: ...
@overload
def slugify(text: str, delim: str = "_", lower: bool = True, *, ascii: Literal[True]) -> bytes: ...
@overload
def slugify(text: str, delim: str, lower: bool, ascii: Literal[True]) -> bytes: ...
@overload
def slugify(text: str, delim: str = "_", lower: bool = True, ascii: Literal[False] = False) -> str: ...
def split_punct_ws(text: str) -> list[str]: ...
def unit_len(sized_iterable: Sized, unit_noun: str = "item") -> str: ...
def ordinalize(number: int | str, ext_only: bool = False) -> str: ...
def cardinalize(unit_noun: str, count: int) -> str: ...
def singularize(word: str) -> str: ...
def pluralize(word: str) -> str: ...
def find_hashtags(string: str) -> list[str]: ...
def a10n(string: str) -> str: ...
def strip_ansi(text: str) -> str: ...
def asciify(text: str | bytes | bytearray, ignore: bool = False) -> bytes: ...
def is_ascii(text: str) -> bool: ...

class DeaccenterDict(dict[int, int]):
    def __missing__(self, key: int) -> int: ...

def bytes2human(nbytes: int, ndigits: int = 0) -> str: ...

class HTMLTextExtractor(HTMLParser):
    strict: bool
    convert_charrefs: bool
    result: list[str]
    def __init__(self) -> None: ...
    def handle_data(self, d: str) -> None: ...
    def handle_charref(self, number: str) -> None: ...
    def handle_entityref(self, name: str) -> None: ...
    def get_text(self) -> str: ...

def html2text(html: str) -> str: ...
def gunzip_bytes(bytestring: ReadableBuffer) -> bytes: ...
def gzip_bytes(bytestring: ReadableBuffer, level: int = 6) -> int: ...
def iter_splitlines(text: str) -> Generator[str, None, None]: ...
def indent(text: str, margin: str, newline: str = "\n", key: Callable[[str], bool] = ...) -> str: ...
def is_uuid(obj, version: int = 4) -> bool: ...
def escape_shell_args(args: Iterable[str], sep: str = " ", style: Literal["cmd", "sh"] | None = None) -> str: ...
def args2sh(args: Iterable[str], sep: str = " ") -> str: ...
def args2cmd(args: Iterable[str], sep: str = " ") -> str: ...
def parse_int_list(range_string: str, delim: str = ",", range_delim: str = "-") -> list[int]: ...
def format_int_list(int_list: list[int], delim: str = ",", range_delim: str = "-", delim_space: bool = False) -> str: ...
def complement_int_list(
    range_string: str, range_start: int = 0, range_end: int | None = None, delim: str = ",", range_delim: str = "-"
) -> str: ...
def int_ranges_from_int_list(range_string: str, delim: str = ",", range_delim: str = "-") -> tuple[tuple[int, int], ...]: ...

class MultiReplace:
    group_map: dict[str, str]
    combined_pattern: Pattern[str]
    def __init__(self, sub_map: dict[str, str], **kwargs) -> None: ...
    def sub(self, text: str) -> str: ...

def multi_replace(text: str, sub_map: dict[str, str], **kwargs) -> str: ...
def unwrap_text(text: str, ending: str | None = "\n\n") -> str: ...
def removeprefix(text: str, prefix: str) -> str: ...

__all__ = [
    "camel2under",
    "under2camel",
    "slugify",
    "split_punct_ws",
    "unit_len",
    "ordinalize",
    "cardinalize",
    "pluralize",
    "singularize",
    "asciify",
    "is_ascii",
    "is_uuid",
    "html2text",
    "strip_ansi",
    "bytes2human",
    "find_hashtags",
    "a10n",
    "gzip_bytes",
    "gunzip_bytes",
    "iter_splitlines",
    "indent",
    "escape_shell_args",
    "args2cmd",
    "args2sh",
    "parse_int_list",
    "format_int_list",
    "complement_int_list",
    "int_ranges_from_int_list",
    "MultiReplace",
    "multi_replace",
    "unwrap_text",
    "removeprefix",
]



from .connectivity import *
from .cuts import *
from .disjoint_paths import *
from .edge_augmentation import *
from .edge_kcomponents import *
from .kcomponents import *
from .kcutsets import *
from .stoerwagner import *
from .utils import *



from logging import Logger

from .base import BaseEndpoint as BaseEndpoint

log: Logger

class ResourceEndpoint(BaseEndpoint):
    def validate_protected_resource_request(self, uri, http_method: str = "GET", body=None, headers=None, realms=None): ...



import socket

from gunicorn.http import Request
from gunicorn.workers import base

from .._types import _AddressType

ALREADY_HANDLED: object

class AsyncWorker(base.Worker):
    worker_connections: int
    alive: bool

    def timeout_ctx(self) -> None: ...
    def is_already_handled(self, respiter: object) -> bool: ...
    def handle(self, listener: socket.socket, client: socket.socket, addr: _AddressType) -> None: ...
    def handle_request(self, listener_name: str, req: Request, sock: socket.socket, addr: _AddressType) -> bool: ...



from _typeshed import Incomplete
from collections.abc import Awaitable, Callable, Iterable, Mapping
from types import TracebackType
from typing import TypeVar

from .models.dummy_entities import DummySegment, DummySubsegment
from .models.segment import Segment, SegmentContextManager
from .models.subsegment import Subsegment, SubsegmentContextManager
from .recorder import AWSXRayRecorder

_T = TypeVar("_T")

class AsyncSegmentContextManager(SegmentContextManager):
    async def __aenter__(self) -> DummySegment | Segment: ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class AsyncSubsegmentContextManager(SubsegmentContextManager):
    async def __call__(
        self, wrapped: Callable[..., Awaitable[_T]], instance, args: Iterable[Incomplete], kwargs: Mapping[str, Incomplete]
    ) -> _T: ...
    async def __aenter__(self) -> DummySubsegment | Subsegment | None: ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class AsyncAWSXRayRecorder(AWSXRayRecorder):
    def capture_async(self, name: str | None = None) -> AsyncSubsegmentContextManager: ...
    def in_segment_async(
        self, name: str | None = None, *, traceid: str | None = None, parent_id: str | None = None, sampling: bool | None = None
    ) -> AsyncSegmentContextManager: ...
    def in_subsegment_async(self, name: str | None = None, *, namespace: str = "local") -> AsyncSubsegmentContextManager: ...
    async def record_subsegment_async(
        self,
        wrapped: Callable[..., Awaitable[_T]],
        instance,
        args: Iterable[Incomplete],
        kwargs: Mapping[str, Incomplete],
        name: str,
        namespace: str,
        meta_processor: Callable[..., object] | None,
    ) -> _T: ...



from collections.abc import Iterable
from re import Pattern
from typing import Any, ClassVar

from markdown.core import Markdown
from markdown.extensions import Extension
from markdown.preprocessors import Preprocessor

class FencedCodeExtension(Extension):
    def __init__(self, **kwargs) -> None: ...

class FencedBlockPreprocessor(Preprocessor):
    FENCED_BLOCK_RE: ClassVar[Pattern[str]]
    checked_for_deps: bool
    codehilite_conf: dict[str, Any]
    use_attr_list: bool
    bool_options: list[str]
    def __init__(self, md: Markdown, config: dict[str, Any]) -> None: ...
    def handle_attrs(self, attrs: Iterable[tuple[str, str]]) -> tuple[str, list[str], dict[str, Any]]: ...

def makeExtension(**kwargs) -> FencedCodeExtension: ...



from _typeshed import Incomplete

from reportlab.graphics.barcode.common import Barcode

class FIM(Barcode):
    barWidth: Incomplete
    spaceWidth: Incomplete
    barHeight: Incomplete
    rquiet: Incomplete
    lquiet: Incomplete
    quiet: int
    def __init__(self, value: str = "", **args) -> None: ...
    valid: int
    validated: str
    def validate(self): ...
    decomposed: str
    def decompose(self): ...
    def computeSize(self) -> None: ...
    def draw(self) -> None: ...

class POSTNET(Barcode):
    quiet: int
    shortHeight: Incomplete
    barHeight: Incomplete
    barWidth: Incomplete
    spaceWidth: Incomplete
    def __init__(self, value: str = "", **args) -> None: ...
    validated: str
    valid: int
    def validate(self): ...
    encoded: str
    def encode(self): ...
    decomposed: str
    def decompose(self): ...
    def computeSize(self) -> None: ...
    def draw(self) -> None: ...



import re
import sys
from collections.abc import Generator

from serial.tools.list_ports_common import ListPortInfo

if sys.platform == "win32":
    from serial.tools.list_ports_windows import comports as comports
else:
    from serial.tools.list_ports_posix import comports as comports

def grep(regexp: str | re.Pattern[str], include_links: bool = False) -> Generator[ListPortInfo]: ...
def main() -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.core.framework.graph_pb2
import tensorflow.core.framework.types_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class OptimizedFunctionGraph(google.protobuf.message.Message):
    """Optimized function graph after instantiation-related graph optimization
    passes (up till before graph partitioning). The first half of the proto is
    representing a GraphDef and the rest of the fields are extra information from
    graph optimizations.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _OptimizationSource:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _OptimizationSourceEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[OptimizedFunctionGraph._OptimizationSource.ValueType],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        SOURCE_UNSPECIFIED: OptimizedFunctionGraph._OptimizationSource.ValueType  # 0
        AOT: OptimizedFunctionGraph._OptimizationSource.ValueType  # 1
        JIT: OptimizedFunctionGraph._OptimizationSource.ValueType  # 2

    class OptimizationSource(_OptimizationSource, metaclass=_OptimizationSourceEnumTypeWrapper):
        """Enum for distinguishing the origin where the proto is created.

        AOT: proto is created in ahead-of-time environment, which can be different
        from the environment where the graph is actually executed.

        JIT: proto is created in just-in-time execution, which has the same
        environment as the one the graph is actually executed.
        """

    SOURCE_UNSPECIFIED: OptimizedFunctionGraph.OptimizationSource.ValueType  # 0
    AOT: OptimizedFunctionGraph.OptimizationSource.ValueType  # 1
    JIT: OptimizedFunctionGraph.OptimizationSource.ValueType  # 2

    @typing.final
    class NodeNameToControlRetEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(self, *, key: builtins.str | None = ..., value: builtins.str | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    FUNCTION_GRAPH_FIELD_NUMBER: builtins.int
    NODE_NAME_TO_CONTROL_RET_FIELD_NUMBER: builtins.int
    RET_TYPES_FIELD_NUMBER: builtins.int
    NUM_RETURN_NODES_FIELD_NUMBER: builtins.int
    SOURCE_FIELD_NUMBER: builtins.int
    OPTIMIZATION_TIME_USECS_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Function name. It can be a human-readable SignatureDef's method name, or a
    FunctionDef name.
    """
    num_return_nodes: builtins.int
    """Number of return nodes. This is an output of graph preprocessing."""
    source: global___OptimizedFunctionGraph.OptimizationSource.ValueType
    """Indicates the source environment where this proto is generated."""
    optimization_time_usecs: builtins.int
    """Time (in microseconds) spent on running the graph optimization passes for
    this function.
    """
    @property
    def function_graph(self) -> tensorflow.core.framework.graph_pb2.GraphDef:
        """Optimized function graph."""

    @property
    def node_name_to_control_ret(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """Maps from node name to control ret. This is an output from running TF/XLA
        bridge.
        """

    @property
    def ret_types(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[tensorflow.core.framework.types_pb2.DataType.ValueType]:
        """Return node types of the function. This is an output of graph
        preprocessing.
        """

    def __init__(
        self,
        *,
        name: builtins.str | None = ...,
        function_graph: tensorflow.core.framework.graph_pb2.GraphDef | None = ...,
        node_name_to_control_ret: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        ret_types: collections.abc.Iterable[tensorflow.core.framework.types_pb2.DataType.ValueType] | None = ...,
        num_return_nodes: builtins.int | None = ...,
        source: global___OptimizedFunctionGraph.OptimizationSource.ValueType | None = ...,
        optimization_time_usecs: builtins.int | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "_optimization_time_usecs",
            b"_optimization_time_usecs",
            "_source",
            b"_source",
            "function_graph",
            b"function_graph",
            "optimization_time_usecs",
            b"optimization_time_usecs",
            "source",
            b"source",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "_optimization_time_usecs",
            b"_optimization_time_usecs",
            "_source",
            b"_source",
            "function_graph",
            b"function_graph",
            "name",
            b"name",
            "node_name_to_control_ret",
            b"node_name_to_control_ret",
            "num_return_nodes",
            b"num_return_nodes",
            "optimization_time_usecs",
            b"optimization_time_usecs",
            "ret_types",
            b"ret_types",
            "source",
            b"source",
        ],
    ) -> None: ...
    @typing.overload
    def WhichOneof(
        self, oneof_group: typing.Literal["_optimization_time_usecs", b"_optimization_time_usecs"]
    ) -> typing.Literal["optimization_time_usecs"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_source", b"_source"]) -> typing.Literal["source"] | None: ...

global___OptimizedFunctionGraph = OptimizedFunctionGraph



import sys

from passlib.context import CryptContext

linux_context: CryptContext
linux2_context: CryptContext
freebsd_context: CryptContext
openbsd_context: CryptContext
netbsd_context: CryptContext
# Only exists if crypt is present
if sys.version_info < (3, 13) and sys.platform != "win32":
    host_context: CryptContext
    __all__ = ["linux_context", "linux2_context", "openbsd_context", "netbsd_context", "freebsd_context", "host_context"]
else:
    __all__ = ["linux_context", "linux2_context", "openbsd_context", "netbsd_context", "freebsd_context"]



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.classes.multidigraph import MultiDiGraph
from networkx.utils.backends import _dispatchable

def write_p2g(G: Graph[_Node], path, encoding: str = "utf-8") -> None: ...
@_dispatchable
def read_p2g(path, encoding: str = "utf-8") -> MultiDiGraph[Incomplete]: ...
@_dispatchable
def parse_p2g(lines) -> MultiDiGraph[Incomplete]: ...



import datetime
from typing import Final, Literal

MON: Final = 0
TUE: Final = 1
WED: Final = 2
THU: Final = 3
FRI: Final = 4
SAT: Final = 5
SUN: Final = 6

def to_jd(year: int, week: int, day: int) -> float: ...
def from_jd(jd: float) -> datetime._IsoCalendarDate: ...
def weeks_per_year(year: int) -> Literal[52, 53]: ...
def from_gregorian(year: int, month: int, day: int) -> datetime._IsoCalendarDate: ...
def to_gregorian(year: int, week: int, day: int) -> tuple[int, int, int]: ...
def format(year: int, week: int, day: int) -> str: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "resource_allocation_index",
    "jaccard_coefficient",
    "adamic_adar_index",
    "preferential_attachment",
    "cn_soundarajan_hopcroft",
    "ra_index_soundarajan_hopcroft",
    "within_inter_cluster",
    "common_neighbor_centrality",
]

@_dispatchable
def resource_allocation_index(G: Graph[_Node], ebunch=None): ...
@_dispatchable
def jaccard_coefficient(G: Graph[_Node], ebunch=None): ...
@_dispatchable
def adamic_adar_index(G: Graph[_Node], ebunch=None): ...
@_dispatchable
def common_neighbor_centrality(G: Graph[_Node], ebunch=None, alpha=0.8): ...
@_dispatchable
def preferential_attachment(G: Graph[_Node], ebunch=None): ...
@_dispatchable
def cn_soundarajan_hopcroft(G: Graph[_Node], ebunch=None, community: str | None = "community"): ...
@_dispatchable
def ra_index_soundarajan_hopcroft(G: Graph[_Node], ebunch=None, community: str | None = "community"): ...
@_dispatchable
def within_inter_cluster(G: Graph[_Node], ebunch=None, delta: float | None = 0.001, community: str | None = "community"): ...



from _typeshed import Incomplete

from braintree.dispute import Dispute
from braintree.error_result import ErrorResult
from braintree.successful_result import SuccessfulResult

class DisputeGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def accept(self, dispute_id: str) -> SuccessfulResult | ErrorResult: ...
    def add_file_evidence(self, dispute_id: str, document_upload_id_or_request) -> SuccessfulResult | ErrorResult | None: ...
    def add_text_evidence(self, dispute_id: str, content_or_request) -> SuccessfulResult | ErrorResult | None: ...
    def finalize(self, dispute_id: str) -> SuccessfulResult | ErrorResult: ...
    def find(self, dispute_id: str) -> Dispute: ...
    def remove_evidence(self, dispute_id: str, evidence_id: int) -> SuccessfulResult | ErrorResult: ...
    search_criteria: dict[Incomplete, Incomplete]
    def search(self, *query) -> SuccessfulResult: ...



from __future__ import annotations

from typing import List
from typing_extensions import assert_type

import regex

# Regression tests for #9263
assert_type(list(regex.finditer(r"foo", "foo")), List[regex.Match[str]])
pat = regex.compile(rb"foo")
assert_type(list(pat.finditer(b"foo")), List[regex.Match[bytes]])



import csv
from _typeshed import Incomplete
from collections.abc import Callable, Sequence
from typing import ClassVar, Final
from typing_extensions import deprecated

from docutils import nodes
from docutils.parsers.rst import Directive

__docformat__: Final = "reStructuredText"

def align(argument): ...

class Table(Directive):
    option_spec: ClassVar[dict[str, Callable[[str], str | list[str]]]]
    def make_title(self): ...
    def check_table_dimensions(self, rows, header_rows, stub_columns) -> None: ...
    def set_table_width(self, table_node) -> None: ...
    @property
    def widths(self): ...
    def get_column_widths(self, n_cols): ...
    def extend_short_rows_with_empty_cells(self, columns, parts) -> None: ...

class RSTTable(Table):
    def run(self) -> Sequence[nodes.table | nodes.system_message]: ...

class CSVTable(Table):
    class DocutilsDialect(csv.Dialect):
        delimiter: str
        quotechar: str
        doublequote: bool
        skipinitialspace: bool
        strict: bool
        lineterminator: str
        quoting: Incomplete
        escapechar: Incomplete
        def __init__(self, options) -> None: ...

    @deprecated("Deprecated and will be removed in Docutils 1.0.")
    class HeaderDialect(csv.Dialect):
        delimiter: str
        quotechar: str
        escapechar: str
        doublequote: bool
        skipinitialspace: bool
        strict: bool
        lineterminator: str
        quoting: Incomplete
        def __init__(self) -> None: ...

    @staticmethod
    @deprecated("Deprecated and not required with Python 3; will be removed in Docutils 0.22.")
    def check_requirements() -> None: ...
    def process_header_option(self): ...
    def run(self) -> Sequence[nodes.table | nodes.system_message]: ...
    def get_csv_data(self): ...
    def parse_csv_data_into_rows(self, csv_data, dialect, source): ...

class ListTable(Table):
    def run(self) -> Sequence[nodes.table | nodes.system_message]: ...
    def check_list_content(self, node): ...
    def build_table_from_list(self, table_data, col_widths, header_rows, stub_columns) -> nodes.table: ...



from collections.abc import Iterable, Sequence
from typing import Any

from django import forms

from .formats.base_formats import Format
from .resources import ModelResource, Resource

class ImportExportFormBase(forms.Form):
    resource: forms.ChoiceField
    format: forms.ChoiceField
    def __init__(
        self, formats: list[type[Format]], resources: list[type[Resource[Any]]] | None = None, **kwargs: Any
    ) -> None: ...

class ImportForm(ImportExportFormBase):
    import_file: forms.FileField
    field_order: Sequence[str]
    @property
    def media(self) -> forms.Media: ...

class ConfirmImportForm(forms.Form):
    import_file_name: forms.CharField
    original_file_name: forms.CharField
    resource: forms.CharField
    def clean_import_file_name(self) -> str: ...

class ExportForm(ImportExportFormBase):
    export_items: forms.MultipleChoiceField

class SelectableFieldsExportForm(ExportForm):
    resources: Iterable[ModelResource[Any]]
    is_selectable_fields_form: bool
    resource_fields: dict[str, list[str]]
    @staticmethod
    def create_boolean_field_name(resource: ModelResource[Any], field_name: str) -> str: ...
    def get_selected_resource(self) -> ModelResource[Any]: ...
    def get_selected_resource_export_fields(self) -> list[str]: ...



from __future__ import annotations

import sys
from pathlib import Path, PureWindowsPath
from typing_extensions import assert_type


class MyCustomPath(Path): ...


if Path("asdf") == Path("asdf"):
    ...

# https://github.com/python/typeshed/issues/10661
# Provide a true positive error when comparing Path to str
# mypy should report a comparison-overlap error with --strict-equality,
# and pyright should report a reportUnnecessaryComparison error
if Path("asdf") == "asdf":  # type: ignore
    ...

# Errors on comparison here are technically false positives. However, this comparison is a little
# interesting: it can never hold true on Posix, but could hold true on Windows. We should experiment
# with more accurate __new__, such that we only get an error for such comparisons on platforms
# where they can never hold true.
if PureWindowsPath("asdf") == Path("asdf"):  # type: ignore
    ...


if sys.version_info >= (3, 13):
    pth = MyCustomPath.from_uri("file:///tmp/abc.txt")
    assert_type(pth, MyCustomPath)


if sys.version_info >= (3, 14):
    pth = MyCustomPath("asdf")
    # With text path, type should be preserved.
    assert_type(pth.move_into("asdf"), MyCustomPath)
    assert_type(pth.move("asdf"), MyCustomPath)
    assert_type(pth.copy("asdf"), MyCustomPath)
    assert_type(pth.copy_into("asdf"), MyCustomPath)

    # With an actual path type, that type should be preserved.
    assert_type(pth.move_into(Path("asdf")), Path)
    assert_type(pth.move(Path("asdf")), Path)
    assert_type(pth.copy(Path("asdf")), Path)
    assert_type(pth.copy_into(Path("asdf")), Path)



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.payment_method_nonce import PaymentMethodNonce
from braintree.successful_result import SuccessfulResult

class PaymentMethodNonceGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def create(self, payment_method_token: str, params=...) -> SuccessfulResult | ErrorResult: ...
    def find(self, payment_method_nonce: str) -> PaymentMethodNonce: ...



from _typeshed import Incomplete

class ClientSecretJWT:
    name: str
    alg: str
    token_endpoint: Incomplete
    claims: Incomplete
    headers: Incomplete
    def __init__(self, token_endpoint=None, claims=None, headers=None, alg=None) -> None: ...
    def sign(self, auth, token_endpoint): ...
    def __call__(self, auth, method, uri, headers, body): ...

class PrivateKeyJWT(ClientSecretJWT):
    name: str
    alg: str
    def sign(self, auth, token_endpoint): ...



from _typeshed import StrPath
from typing import IO, AnyStr
from typing_extensions import Self

__tracebackhide__: bool

def contents_of(file: IO[AnyStr] | StrPath, encoding: str = "utf-8") -> str: ...

class FileMixin:
    def exists(self) -> Self: ...
    def does_not_exist(self) -> Self: ...
    def is_file(self) -> Self: ...
    def is_directory(self) -> Self: ...
    def is_named(self, filename: str) -> Self: ...
    def is_child_of(self, parent: str) -> Self: ...



from networkx.classes.digraph import DiGraph
from networkx.classes.graph import _Node
from networkx.utils.backends import _dispatchable

__all__ = ["stochastic_graph"]

@_dispatchable
def stochastic_graph(G: DiGraph[_Node], copy: bool = True, weight: str = "weight"): ...



from authlib.oauth2 import OAuth2Error

class InteractionRequiredError(OAuth2Error):
    error: str

class LoginRequiredError(OAuth2Error):
    error: str

class AccountSelectionRequiredError(OAuth2Error):
    error: str

class ConsentRequiredError(OAuth2Error):
    error: str

class InvalidRequestURIError(OAuth2Error):
    error: str

class InvalidRequestObjectError(OAuth2Error):
    error: str

class RequestNotSupportedError(OAuth2Error):
    error: str

class RequestURINotSupportedError(OAuth2Error):
    error: str

class RegistrationNotSupportedError(OAuth2Error):
    error: str



from typing import Any, List, Optional, Set, Tuple, Type, Union

from cattrs._compat import ExceptionGroup


class StructureHandlerNotFoundError(Exception):
    """
    Error raised when structuring cannot find a handler for converting inputs into
    :attr:`type_`.
    """

    def __init__(self, message: str, type_: Type) -> None:
        super().__init__(message)
        self.type_ = type_


class BaseValidationError(ExceptionGroup):
    cl: Type

    def __new__(cls, message, excs, cl: Type):
        obj = super().__new__(cls, message, excs)
        obj.cl = cl
        return obj

    def derive(self, excs):
        return ClassValidationError(self.message, excs, self.cl)


class IterableValidationNote(str):
    """Attached as a note to an exception when an iterable element fails structuring."""

    index: Union[int, str]  # Ints for list indices, strs for dict keys
    type: Any

    def __new__(
        cls, string: str, index: Union[int, str], type: Any
    ) -> "IterableValidationNote":
        instance = str.__new__(cls, string)
        instance.index = index
        instance.type = type
        return instance

    def __getnewargs__(self) -> Tuple[str, Union[int, str], Any]:
        return (str(self), self.index, self.type)


class IterableValidationError(BaseValidationError):
    """Raised when structuring an iterable."""

    def group_exceptions(
        self,
    ) -> Tuple[List[Tuple[Exception, IterableValidationNote]], List[Exception]]:
        """Split the exceptions into two groups: with and without validation notes."""
        excs_with_notes = []
        other_excs = []
        for subexc in self.exceptions:
            if hasattr(subexc, "__notes__"):
                for note in subexc.__notes__:
                    if note.__class__ is IterableValidationNote:
                        excs_with_notes.append((subexc, note))
                        break
                else:
                    other_excs.append(subexc)
            else:
                other_excs.append(subexc)

        return excs_with_notes, other_excs


class AttributeValidationNote(str):
    """Attached as a note to an exception when an attribute fails structuring."""

    name: str
    type: Any

    def __new__(cls, string: str, name: str, type: Any) -> "AttributeValidationNote":
        instance = str.__new__(cls, string)
        instance.name = name
        instance.type = type
        return instance

    def __getnewargs__(self) -> Tuple[str, str, Any]:
        return (str(self), self.name, self.type)


class ClassValidationError(BaseValidationError):
    """Raised when validating a class if any attributes are invalid."""

    def group_exceptions(
        self,
    ) -> Tuple[List[Tuple[Exception, AttributeValidationNote]], List[Exception]]:
        """Split the exceptions into two groups: with and without validation notes."""
        excs_with_notes = []
        other_excs = []
        for subexc in self.exceptions:
            if hasattr(subexc, "__notes__"):
                for note in subexc.__notes__:
                    if note.__class__ is AttributeValidationNote:
                        excs_with_notes.append((subexc, note))
                        break
                else:
                    other_excs.append(subexc)
            else:
                other_excs.append(subexc)

        return excs_with_notes, other_excs


class ForbiddenExtraKeysError(Exception):
    """
    Raised when `forbid_extra_keys` is activated and such extra keys are detected
    during structuring.

    The attribute `extra_fields` is a sequence of those extra keys, which were the
    cause of this error, and `cl` is the class which was structured with those extra
    keys.
    """

    def __init__(
        self, message: Optional[str], cl: Type, extra_fields: Set[str]
    ) -> None:
        self.cl = cl
        self.extra_fields = extra_fields
        cln = cl.__name__

        super().__init__(
            message
            or f"Extra fields in constructor for {cln}: {', '.join(extra_fields)}"
        )



from _typeshed import Incomplete, StrPath
from collections.abc import Callable
from re import Pattern
from typing import ClassVar, Final, NoReturn

from docutils import nodes, writers
from docutils.frontend import Values
from docutils.languages import _LanguageModule

__docformat__: Final = "reStructuredText"

class Writer(writers.Writer[str]):
    settings_defaults: ClassVar[dict[str, str]]
    relative_path_settings: ClassVar[tuple[str, ...]]
    config_section: ClassVar[str]
    config_section_dependencies: ClassVar[tuple[str, ...]]
    visitor_attributes: ClassVar[tuple[str, ...]]
    visitor: nodes.NodeVisitor
    def translate(self) -> None: ...
    def apply_template(self) -> str: ...
    def interpolation_dict(self) -> dict[str, str]: ...
    def assemble_parts(self) -> None: ...

class HTMLTranslator(nodes.NodeVisitor):
    doctype: ClassVar[str]
    doctype_mathml: ClassVar[str]
    head_prefix_template: ClassVar[str]
    content_type: ClassVar[str]
    generator: ClassVar[str]
    documenttag_args: ClassVar[dict[str, str]]
    mathjax_script: ClassVar[str]
    mathjax_url: ClassVar[str]
    stylesheet_link: ClassVar[str]
    embedded_stylesheet: ClassVar[str]
    words_and_spaces: ClassVar[Pattern[str]]
    in_word_wrap_point: ClassVar[Pattern[str]]
    lang_attribute: ClassVar[str]
    special_characters: ClassVar[dict[int, str]]
    videotypes: ClassVar[tuple[str, ...]]
    attribution_formats: ClassVar[dict[str, tuple[str, str]]]
    settings: Values
    language: _LanguageModule
    initial_header_level: int
    image_loading: str
    body: list[str]
    body_prefix: list[str]
    body_pre_docinfo: list[Incomplete]
    body_suffix: list[str]
    docinfo: list[str]
    footer: list[str]
    fragment: list[Incomplete]
    head: list[Incomplete]
    head_prefix: list[Incomplete]
    header: list[Incomplete]
    html_body: list[Incomplete]
    html_head: list[Incomplete]
    html_prolog: list[Incomplete]
    html_subtitle: list[Incomplete]
    html_title: list[Incomplete]
    meta: list[Incomplete]
    stylesheet: list[Incomplete]
    title: list[Incomplete]
    subtitle: list[Incomplete]
    context: list[Incomplete]
    section_level: int
    colspecs: list[Incomplete]
    compact_p: bool
    compact_simple: bool
    compact_field_list: bool
    in_docinfo: bool
    in_sidebar: bool
    in_document_title: int
    in_mailto: bool
    author_in_authors: bool
    math_header: list[str]
    messages: list[Incomplete]
    def __init__(self, document: nodes.document) -> None: ...
    def astext(self) -> str: ...
    def attval(self, text: str, whitespace: Pattern[str] = ...) -> str: ...
    def cloak_email(self, addr: str) -> str: ...
    def cloak_mailto(self, uri: str) -> str: ...
    def encode(self, text: object) -> str: ...
    def image_size(self, node: nodes.image) -> str: ...
    def read_size_with_PIL(self, node) -> tuple[int, int] | None: ...
    def prepare_svg(self, code: str | bytes, node: nodes.Element, atts: dict[str, Incomplete]) -> str: ...
    def stylesheet_call(self, path: StrPath, adjust_path: bool | None = None) -> str: ...
    def starttag(self, node: nodes.Element, tagname: str, suffix: str = "\n", empty: bool = False, **attributes) -> str: ...
    def emptytag(self, node: nodes.Element, tagname: str, suffix: str = "\n", **attributes) -> str: ...
    def report_messages(self, node: nodes.Node) -> None: ...
    def set_class_on_child(self, node, class_, index: int = 0) -> None: ...
    def visit_Text(self, node: nodes.Text) -> None: ...
    def depart_Text(self, node: nodes.Text) -> None: ...
    def visit_abbreviation(self, node: nodes.abbreviation) -> None: ...
    def depart_abbreviation(self, node: nodes.abbreviation) -> None: ...
    def visit_acronym(self, node: nodes.acronym) -> None: ...
    def depart_acronym(self, node: nodes.acronym) -> None: ...
    def visit_address(self, node: nodes.address) -> None: ...
    def depart_address(self, node: nodes.address) -> None: ...
    def visit_admonition(self, node: nodes.admonition) -> None: ...
    def depart_admonition(self, node: nodes.admonition | None = None) -> None: ...
    def visit_attribution(self, node: nodes.attribution) -> None: ...
    def depart_attribution(self, node: nodes.attribution) -> None: ...
    def visit_author(self, node: nodes.author) -> None: ...
    def depart_author(self, node: nodes.author) -> None: ...
    def visit_authors(self, node: nodes.authors) -> None: ...
    def depart_authors(self, node: nodes.authors) -> None: ...
    def visit_block_quote(self, node: nodes.block_quote) -> None: ...
    def depart_block_quote(self, node: nodes.block_quote) -> None: ...
    def check_simple_list(self, node: nodes.Node) -> bool: ...
    def is_compactable(self, node: nodes.Element) -> bool: ...
    def visit_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def depart_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def visit_caption(self, node: nodes.caption) -> None: ...
    def depart_caption(self, node: nodes.caption) -> None: ...
    def visit_citation(self, node: nodes.citation) -> None: ...
    def depart_citation(self, node: nodes.citation) -> None: ...
    def visit_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def depart_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def visit_classifier(self, node: nodes.classifier) -> None: ...
    def depart_classifier(self, node: nodes.classifier) -> None: ...
    def visit_colspec(self, node: nodes.colspec) -> None: ...
    def depart_colspec(self, node: nodes.colspec) -> None: ...
    def visit_comment(self, node: nodes.comment, sub: Callable[[str, str], str] = ...) -> None: ...
    def visit_compound(self, node: nodes.compound) -> None: ...
    def depart_compound(self, node: nodes.compound) -> None: ...
    def visit_container(self, node: nodes.container) -> None: ...
    def depart_container(self, node: nodes.container) -> None: ...
    def visit_contact(self, node: nodes.contact) -> None: ...
    def depart_contact(self, node: nodes.contact) -> None: ...
    def visit_copyright(self, node: nodes.copyright) -> None: ...
    def depart_copyright(self, node: nodes.copyright) -> None: ...
    def visit_date(self, node: nodes.date) -> None: ...
    def depart_date(self, node: nodes.date) -> None: ...
    def visit_decoration(self, node: nodes.decoration) -> None: ...
    def depart_decoration(self, node: nodes.decoration) -> None: ...
    def visit_definition(self, node: nodes.definition) -> None: ...
    def depart_definition(self, node: nodes.definition) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...
    def depart_definition_list(self, node: nodes.definition_list) -> None: ...
    def visit_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def depart_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def visit_description(self, node: nodes.description) -> None: ...
    def depart_description(self, node: nodes.description) -> None: ...
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...
    def depart_docinfo(self, node: nodes.docinfo) -> None: ...
    def visit_docinfo_item(self, node, name: str, meta: bool = True) -> None: ...
    def depart_docinfo_item(self) -> None: ...
    def visit_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def depart_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def visit_document(self, node: nodes.document) -> None: ...
    def depart_document(self, node: nodes.document) -> None: ...
    def visit_emphasis(self, node: nodes.emphasis) -> None: ...
    def depart_emphasis(self, node: nodes.emphasis) -> None: ...
    def visit_entry(self, node: nodes.entry) -> None: ...
    def depart_entry(self, node: nodes.entry) -> None: ...
    def visit_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def depart_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def visit_field_list(self, node: nodes.field_list) -> None: ...
    def depart_field_list(self, node: nodes.field_list) -> None: ...
    def visit_field(self, node: nodes.field) -> None: ...
    def depart_field(self, node: nodes.field) -> None: ...
    def visit_field_name(self, node: nodes.field_name) -> None: ...
    def depart_field_name(self, node: nodes.field_name) -> None: ...
    def visit_field_body(self, node: nodes.field_body) -> None: ...
    def depart_field_body(self, node: nodes.field_body) -> None: ...
    def visit_figure(self, node: nodes.figure) -> None: ...
    def depart_figure(self, node: nodes.figure) -> None: ...
    def visit_footer(self, node: nodes.footer) -> None: ...
    def depart_footer(self, node: nodes.footer) -> None: ...
    def visit_footnote(self, node: nodes.footnote) -> None: ...
    def depart_footnote(self, node: nodes.footnote) -> None: ...
    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def depart_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def visit_generated(self, node: nodes.generated) -> None: ...
    def depart_generated(self, node: nodes.generated) -> None: ...
    def visit_header(self, node: nodes.header) -> None: ...
    def depart_header(self, node: nodes.header) -> None: ...
    def visit_image(self, node: nodes.image) -> None: ...
    def depart_image(self, node: nodes.image) -> None: ...
    def visit_inline(self, node: nodes.inline) -> None: ...
    def depart_inline(self, node: nodes.inline) -> None: ...
    def visit_label(self, node: nodes.label) -> None: ...
    def depart_label(self, node: nodes.label) -> None: ...
    def visit_legend(self, node: nodes.legend) -> None: ...
    def depart_legend(self, node: nodes.legend) -> None: ...
    def visit_line(self, node: nodes.line) -> None: ...
    def depart_line(self, node: nodes.line) -> None: ...
    def visit_line_block(self, node: nodes.line_block) -> None: ...
    def depart_line_block(self, node: nodes.line_block) -> None: ...
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def depart_list_item(self, node: nodes.list_item) -> None: ...
    def visit_literal(self, node: nodes.literal) -> None: ...
    def depart_literal(self, node: nodes.literal) -> None: ...
    def visit_literal_block(self, node: nodes.literal_block) -> None: ...
    def depart_literal_block(self, node: nodes.literal_block) -> None: ...
    math_tags: dict[str, tuple[str, str, list[str]]]
    math_output: str | Incomplete
    def visit_math(self, node: nodes.math) -> None: ...
    def depart_math(self, node: nodes.math) -> None: ...
    def visit_math_block(self, node: nodes.math_block) -> None: ...
    def depart_math_block(self, node: nodes.math_block) -> None: ...
    def visit_meta(self, node: nodes.meta) -> None: ...
    def depart_meta(self, node: nodes.meta) -> None: ...
    def visit_option(self, node: nodes.option) -> None: ...
    def depart_option(self, node: nodes.option) -> None: ...
    def visit_option_argument(self, node: nodes.option_argument) -> None: ...
    def depart_option_argument(self, node: nodes.option_argument) -> None: ...
    def visit_option_group(self, node: nodes.option_group) -> None: ...
    def depart_option_group(self, node: nodes.option_group) -> None: ...
    def visit_option_list(self, node: nodes.option_list) -> None: ...
    def depart_option_list(self, node: nodes.option_list) -> None: ...
    def visit_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def depart_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def visit_option_string(self, node: nodes.option_string) -> None: ...
    def depart_option_string(self, node: nodes.option_string) -> None: ...
    def visit_organization(self, node: nodes.organization) -> None: ...
    def depart_organization(self, node: nodes.organization) -> None: ...
    def visit_paragraph(self, node: nodes.paragraph) -> None: ...
    def depart_paragraph(self, node: nodes.paragraph) -> None: ...
    def visit_problematic(self, node: nodes.problematic) -> None: ...
    def depart_problematic(self, node: nodes.problematic) -> None: ...
    def visit_raw(self, node: nodes.raw) -> None: ...
    def visit_reference(self, node: nodes.reference) -> None: ...
    def depart_reference(self, node: nodes.reference) -> None: ...
    def visit_revision(self, node: nodes.revision) -> None: ...
    def depart_revision(self, node: nodes.revision) -> None: ...
    def visit_row(self, node: nodes.row) -> None: ...
    def depart_row(self, node: nodes.row) -> None: ...
    def visit_rubric(self, node: nodes.rubric) -> None: ...
    def depart_rubric(self, node: nodes.rubric) -> None: ...
    def visit_section(self, node: nodes.section) -> None: ...
    def depart_section(self, node: nodes.section) -> None: ...
    def visit_sidebar(self, node: nodes.sidebar) -> None: ...
    def depart_sidebar(self, node: nodes.sidebar) -> None: ...
    def visit_status(self, node: nodes.status) -> None: ...
    def depart_status(self, node: nodes.status) -> None: ...
    def visit_strong(self, node: nodes.strong) -> None: ...
    def depart_strong(self, node: nodes.strong) -> None: ...
    def visit_subscript(self, node: nodes.subscript) -> None: ...
    def depart_subscript(self, node: nodes.subscript) -> None: ...
    def visit_substitution_definition(self, node: nodes.substitution_definition) -> None: ...
    def visit_substitution_reference(self, node: nodes.substitution_reference) -> None: ...
    def visit_subtitle(self, node: nodes.subtitle) -> None: ...
    def depart_subtitle(self, node: nodes.subtitle) -> None: ...
    def visit_superscript(self, node: nodes.superscript) -> None: ...
    def depart_superscript(self, node: nodes.superscript) -> None: ...
    def visit_system_message(self, node: nodes.system_message) -> None: ...
    def depart_system_message(self, node: nodes.system_message) -> None: ...
    def visit_table(self, node: nodes.table) -> None: ...
    def depart_table(self, node: nodes.table) -> None: ...
    def visit_target(self, node: nodes.target) -> None: ...
    def depart_target(self, node: nodes.target) -> None: ...
    def visit_tbody(self, node: nodes.tbody) -> None: ...
    def depart_tbody(self, node: nodes.tbody) -> None: ...
    def visit_term(self, node: nodes.term) -> None: ...
    def depart_term(self, node: nodes.term) -> None: ...
    def visit_tgroup(self, node: nodes.tgroup) -> None: ...
    def depart_tgroup(self, node: nodes.tgroup) -> None: ...
    def visit_thead(self, node: nodes.thead) -> None: ...
    def depart_thead(self, node: nodes.thead) -> None: ...
    def section_title_tags(self, node: nodes.Element) -> tuple[str, str]: ...
    def visit_title(self, node: nodes.title) -> None: ...
    def depart_title(self, node: nodes.title) -> None: ...
    def visit_title_reference(self, node: nodes.title_reference) -> None: ...
    def depart_title_reference(self, node: nodes.title_reference) -> None: ...
    def visit_topic(self, node: nodes.topic) -> None: ...
    def depart_topic(self, node: nodes.topic) -> None: ...
    def visit_transition(self, node: nodes.transition) -> None: ...
    def depart_transition(self, node: nodes.transition) -> None: ...
    def visit_version(self, node: nodes.version) -> None: ...
    def depart_version(self, node: nodes.version) -> None: ...
    def unimplemented_visit(self, node: nodes.Node) -> NoReturn: ...

class SimpleListChecker(nodes.GenericNodeVisitor):
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def pass_node(self, node: nodes.Node) -> None: ...
    def ignore_node(self, node: nodes.Node) -> None: ...
    def visit_Text(self, node: nodes.Text) -> None: ...
    def visit_paragraph(self, node: nodes.paragraph) -> None: ...
    def visit_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def visit_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...
    def visit_author(self, node: nodes.author) -> None: ...
    def visit_authors(self, node: nodes.authors) -> None: ...
    def visit_address(self, node: nodes.address) -> None: ...
    def visit_contact(self, node: nodes.contact) -> None: ...
    def visit_copyright(self, node: nodes.copyright) -> None: ...
    def visit_date(self, node: nodes.date) -> None: ...
    def visit_organization(self, node: nodes.organization) -> None: ...
    def visit_status(self, node: nodes.status) -> None: ...
    def visit_version(self, node: nodes.version) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...
    def visit_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def visit_term(self, node: nodes.term) -> None: ...
    def visit_classifier(self, node: nodes.classifier) -> None: ...
    def visit_definition(self, node: nodes.definition) -> None: ...
    def visit_field_list(self, node: nodes.field_list) -> None: ...
    def visit_field(self, node: nodes.field) -> None: ...
    def visit_field_body(self, node: nodes.field_body) -> None: ...
    def visit_field_name(self, node: nodes.field_name) -> None: ...
    def visit_comment(self, node: nodes.comment) -> None: ...
    def visit_substitution_definition(self, node: nodes.substitution_definition) -> None: ...
    def visit_target(self, node: nodes.target) -> None: ...
    def visit_pending(self, node: nodes.pending) -> None: ...



from collections.abc import Iterator
from typing import Protocol

import gdb

class SymValueWrapper(Protocol):
    def symbol(self) -> gdb.Symbol | str: ...
    def value(self) -> gdb._ValueOrNative | None: ...

class _FrameDecoratorBase:
    def __init__(self, base: gdb.Frame | FrameDecorator) -> None: ...
    def elided(self) -> Iterator[gdb.Frame] | None: ...
    def function(self) -> str | None: ...
    def address(self) -> int | None: ...
    def line(self) -> int | None: ...
    def frame_args(self) -> Iterator[SymValueWrapper] | None: ...
    def frame_locals(self) -> Iterator[SymValueWrapper] | None: ...
    def inferior_frame(self) -> gdb.Frame: ...

class FrameDecorator(_FrameDecoratorBase):
    def filename(self) -> str | None: ...

class DAPFrameDecorator(_FrameDecoratorBase):
    def filename(self) -> str | None: ...
